Package: ConversationAlign
Type: Package
Title: Process Text and Compute Linguistic Alignment in Conversation
        Transcripts
Version: 0.4.1
Authors@R: c(
     person("Jamie", "Reilly", 
           role = c("aut", "cre"), 
           email = "jamie_reilly@temple.edu",
           comment = c(ORCID = "0000-0002-0891-438X")),
    person("Virginia", "Ulichney", role = "aut"),
    person("Ben", "Sacks", role = "aut"),
    person("Sarah", "Weinstein", role = "ctb"),
    person("Chelsea", "Helion", role = "ctb"),
    person("Gus", "Cooney", role = "ctb"))
Maintainer: Jamie Reilly <jamie_reilly@temple.edu>
Description: Imports conversation transcripts into R, concatenates them into a single dataframe appending event identifiers, cleans and formats the text, then yokes user-specified psycholinguistic database values to each word.  'ConversationAlign' then computes alignment indices between two interlocutors across each transcript for >40 possible semantic, lexical, and affective dimensions. In addition to alignment, 'ConversationAlign' also produces a table of analytics (e.g., token count, type-token-ratio) in a summary table describing your particular text corpus.
License: LGPL (>= 3)
Encoding: UTF-8
Depends: R (>= 3.5)
Imports: DescTools, dplyr (>= 0.4.3), httr, magrittr, purrr, rlang,
        stringi, stringr, textstem, tibble, tidyr, tidyselect, stats,
        utils, zoo
Suggests: devtools, knitr, rmarkdown, testthat (>= 3.0.0)
URL: https://github.com/Reilly-ConceptsCognitionLab/ConversationAlign
RoxygenNote: 7.3.3
LazyData: true
VignetteBuilder: knitr
Collate: 'ConversationAlign-package.R' 'compute_auc.R'
        'compute_lagcorr.R' 'corpus_analytics.R' 'data.R'
        'generate_shams.R' 'globals.R' 'prep_dyads.R' 'read_1file.R'
        'read_dyads.R' 'replacements_25.R' 'summarize_dyads.R'
        'utils.R' 'zzz.R'
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-29 13:17:50 UTC; Jamie
Author: Jamie Reilly [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0891-438X>),
  Virginia Ulichney [aut],
  Ben Sacks [aut],
  Sarah Weinstein [ctb],
  Chelsea Helion [ctb],
  Gus Cooney [ctb]
Repository: CRAN
Date/Publication: 2026-04-29 15:10:25 UTC
Built: R 4.6.0; ; 2026-04-29 16:03:04 UTC; unix

# Generated by roxygen2: do not edit by hand

export(corpus_analytics)
export(generate_shams)
export(prep_dyads)
export(read_1file)
export(read_dyads)
export(summarize_dyads)
importFrom(DescTools,AUC)
importFrom(dplyr,across)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,consecutive_id)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,lag)
importFrom(dplyr,left_join)
importFrom(dplyr,matches)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,n_distinct)
importFrom(dplyr,na_if)
importFrom(dplyr,rename)
importFrom(dplyr,rename_at)
importFrom(dplyr,rename_with)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(httr,GET)
importFrom(magrittr,"%>%")
importFrom(purrr,map_dfr)
importFrom(rlang,":=")
importFrom(rlang,sym)
importFrom(stats,cor.test)
importFrom(stats,na.omit)
importFrom(stats,sd)
importFrom(stringi,stri_replace_all_fixed)
importFrom(stringi,stri_replace_all_regex)
importFrom(stringr,str_c)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_squish)
importFrom(stringr,str_subset)
importFrom(stringr,str_trim)
importFrom(textstem,lemmatize_strings)
importFrom(tibble,tibble)
importFrom(tidyr,drop_na)
importFrom(tidyr,fill)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,separate)
importFrom(tidyr,separate_rows)
importFrom(tidyselect,all_of)
importFrom(tidyselect,any_of)
importFrom(tidyselect,contains)
importFrom(tidyselect,ends_with)
importFrom(tidyselect,everything)
importFrom(tidyselect,starts_with)
importFrom(tidyselect,where)
importFrom(tools,R_user_dir)
importFrom(utils,download.file)
importFrom(utils,globalVariables)
importFrom(utils,read.csv)
importFrom(utils,select.list)
importFrom(utils,tail)
importFrom(zoo,na.approx)

<!-- README.md is generated from README.Rmd. Please edit that file -->

<br> <br> <br>

<h1 style="color: #8B0000;">

ConversationAlign
</h1>

Open-source software for computing main effects and indices of alignment
across coversation partners in dyadic conversation transcripts.

# <a href="https://reilly-conceptscognitionlab.github.io/ConversationAlign/"><img src="man/figures/logo.png" align="left" height="138" alt="ConversationAlign website" style="margin-right: 20px; margin-bottom: 20px;" /></a>

<div style="clear: both; margin-top: 30px;">

</div>

<div style="display: flex; gap: 10px; flex-wrap: wrap; margin-bottom: 15px;">

[![GitHub
release](https://img.shields.io/github/v/release/Reilly-ConceptsCognitionLab/ConversationAlign?color=blue&include_prereleases&label=Release)](https://github.com/Reilly-ConceptsCognitionLab/ConversationAlign/releases)[![status](https://joss.theoj.org/papers/839ab720504a5c966b4b2893f78ec2b2/status.svg)](https://joss.theoj.org/papers/839ab720504a5c966b4b2893f78ec2b2)

[![GitHub
stars](https://img.shields.io/github/stars/Reilly-ConceptsCognitionLab/ConversationAlign?style=social)](https://github.com/Reilly-ConceptsCognitionLab/ConversationAlign/stargazers)

[![Maintenance](https://img.shields.io/badge/Maintained%3F-yes-green.svg)](https://github.com/Reilly-ConceptsCognitionLab/ConversationAlign/graphs/commit-activity)
[![License: GPL
v3+](https://img.shields.io/badge/License-GPL%20v3+-blue.svg)](https://www.gnu.org/licenses/gpl-3.0)
[![R-CMD-check](https://github.com/Reilly-ConceptsCognitionLab/ConversationAlign/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/Reilly-ConceptsCognitionLab/ConversationAlign/actions/workflows/R-CMD-check.yaml)

</div>

<img src="man/figures/convo_demo.jpg" 
     alt="illustration of processing pipeline used by ConversationAlign" 
     style="float: right; width: 50%; margin-left: 20px; margin-bottom: 10px;" />

# Overview

`ConversationAlign` analyzes alignment and computes main effects across
more than 40 unique dimensions between interlocutors (conversation
partners) engaged in two-person conversations. ConversationAlign
transforms raw language data into simultaneous time series objects
across \>40 possible dimensions via an embedded lookup database. There
are a number of issues you should consider and steps you should take to
prepare your data. <br>

# License

`ConversationAlign` is licensed under the [GNU LGPL
v3.0](https://www.gnu.org/licenses/lgpl-3.0). <br>

# Installation and Technical Considerations

One of the main features of the `ConversationAlign` algorithm involves
yoking norms for many different lexical, affective, and semantic
dimensions to each content word in your conversation transcripts of
interest. We accomplish this by joining your data to several large
lookup databases. These databases are too large to embed within
`ConversationAlign`. When you load `ConversationAlign`, all of these
databases should automatically download and load from an external
companionn repository `ConversationAlign_Data`. `ConversationAlign`
needs these data, so you will need a decent internet connection to load
the package. It might take a second or two to complete the download if
Github is acting up. Install the development version of
ConversationAlign from [GitHub](https://github.com/) using the
`devtools` package.

``` r
# Check if devtools is installed, if not install it
if (!require("devtools", quietly = TRUE)) {
  install.packages("devtools")
}

# Load devtools
library(devtools)

# Check if ConversationAlign is installed, if not install from GitHub
if (!require("ConversationAlign", quietly = TRUE)) {
  devtools::install_github("Reilly-ConceptsCognitionLab/ConversationAlign")
}

# Load SemanticDistance
library(ConversationAlign)
```

# Step 1: Read and Format Transcript Options

## `read_dyads()`

- Reads transcripts from a local drive or directory of your choice.
- Store each of your individual conversation transcripts (`.csv`,
  `.txt`, `.ai`) that you wish to concatenate into a corpus in a folder.
  `ConversationAlign` will search for a folder called `my_transcripts`
  in the same directory as your script. However, feel free to name your
  folder anything you like. You can specify a custom path as an argument
  to `read_dyads()`
- Each transcript must nominally contain two columns of data
  (Participant and Text). All other columns (e.g., meta-data) will be
  retained.

### <span style="color: darkred;">Arguments to `read_dyads`:</span> <br>

- `my_path` default is ‘my_transcripts’, change path to your folder
  name<br>

``` r
#will search for folder 'my_transcripts' in your current directory
MyConvos <- read_dyads()

#will scan custom folder called 'MyStuff' in your current directory, concatenating all files in that folder into a single dataframe
MyConvos2 <- read_dyads(my_path='/MyStuff')
```

<br>

## `read_1file()`

- Read single transcript already in R environment. We will use
  read_1file() to prep the Marc Maron and Terry Gross transcript. Look
  at how the column headers have changed and the object name
  (MaronGross_2013) is now the Event_ID (a document identifier), <br>

### <span style="color: darkred;">Arguments to `read_1file`:</span> <br>

- `my_dat` object already in your R environment containing text and
  speaker information.

``` r
MaryLittleLamb <- read_1file(MaronGross_2013)
#print first ten rows of header
knitr::kable(head(MaronGross_2013, 10), format = "pipe")
```

| speaker | text |
|:---|:---|
| MARON | I’m a little nervous but I’ve prepared I’ve written things on a piece of paper |
| MARON | I don’t know how you prepare I could ask you that - maybe I will But this is how I prepare - I panic |
| MARON | For a while |
| GROSS | Yeah |
| MARON | And then I scramble and then I type some things up and then I handwrite things that are hard to read So I can you know challenge myself on that level during the interview |
| GROSS | Being self-defeating is always a good part of preparation |
| MARON | What is? |
| GROSS | Being self-defeating |
| MARON | Yes |
| GROSS | Self-sabotage |

<br> <br>

# Step 2: Clean, Format, Align Norms

## `prep_dyads()`

-Cleans, formats, and vectorizes conversation transwcripts to a
one-word-per-row format -Yokes psycholinguistic norms for up to three
dimensions at a time (from \<40 possible dimensions) to each content
word. -Retains metadata

### <span style="color: darkred;">Arguments to `prep_dyads()`:</span> <br>

- `dat_read` name of the dataframe created during `read_dyads()` <br>
- `omit_stops` T/F (default=T) option to remove stopwords
- `lemmatize` T/F (default=T) lemmatize strings converting each entry to
  its dictionary form
- `which_stoplist` quoted argument specifying stopword list to apply,
  options include `none`, `MIT_stops`, `SMART_stops`,
  `CA_OriginalStops`, or `Temple_stops25`. Default is `Temple_stops25`.

``` r
NurseryRhymes_Prepped <- prep_dyads(dat_read=NurseryRhymes, lemmatize=TRUE, omit_stops=T, which_stoplist="Temple_stops25")
```

Example of a prepped dataset embedded as external data in the package
with ‘anger’ values yoked to each word.

``` r
knitr::kable(head(NurseryRhymes_Prepped, 10), format = "simple", digits=2)
```

| Event_ID | Participant_ID | Exchange_Count | Turn_Count | Text_Prep | Text_Clean | emo_anger |
|:---|:---|---:|---:|:---|:---|---:|
| ItsySpider | Yin | 1 | 1 | the | NA | NA |
| ItsySpider | Yin | 1 | 1 | itsy | itsy | -0.02 |
| ItsySpider | Yin | 1 | 1 | bitsy | bitsy | -0.02 |
| ItsySpider | Yin | 1 | 1 | spider | spider | 0.04 |
| ItsySpider | Yin | 1 | 1 | climbed | climb | -0.09 |
| ItsySpider | Yin | 1 | 1 | up | up | -0.06 |
| ItsySpider | Yin | 1 | 1 | the | NA | NA |
| ItsySpider | Yin | 1 | 1 | water | water | -0.17 |
| ItsySpider | Yin | 1 | 1 | spout | spout | 0.05 |
| ItsySpider | Maya | 1 | 2 | down | down | 0.03 |

<br> <br>

# Step 3: Summarize Data, Alignment Stats

## `summarize_dyads()`

This is the computational stage where the package generates a dataframe
boiled down to two rows per converation with summary data appended to
each level of Participant_ID. This returns the difference time series
AUC (dAUC) for every variable of interest you specified and the
correlation at lags -2,,0, 2. You decide whether you want a Pearson or
Spearman lagged correlation. <br>

### <span style="color: darkred;">Arguments to `summarize_dyads()`:</span> <br>

- `df_prep` dataframe created by `prep_dyads()` function
- `custom_lags` user specifies a custom set of turn-lags. Default is
  NULL with `ConversationAlign` producing correlations at a lead of 2
  turns, immediate response, and lag of 2 turns for each dimension of
  interest. <br>
- `sumdat_only` default is TRUE, produces grouped summary dataframe with
  averages by conversation and participant for each alignment dimension,
  FALSE retrains all of the original rows, filling down empty rows of
  summary statistics for the conversation (e.g., AUC)
- `corr_type` specifies correlation madel (parametric default =
  ‘Pearson’); other option ‘Spearman’ for computing turn-by-turn
  correlations across interlocutors for each dimension of interest.

``` r
MarySumDat <- summarize_dyads(df_prep = NurseryRhymes_Prepped, custom_lags=NULL, sumdat_only = TRUE, corr_type='Pearson') 
colnames(MarySumDat)
#>  [1] "Event_ID"           "Participant_ID"     "Dimension"         
#>  [4] "Dimension_Mean"     "AUC_raw"            "AUC_scaled100"     
#>  [7] "Talked_First"       "TurnCorr_Lead2"     "TurnCorr_Immediate"
#> [10] "TurnCorr_Lag2"
knitr::kable(head(MarySumDat, 10), format = "simple", digits = 3)
```

| Event_ID | Participant_ID | Dimension | Dimension_Mean | AUC_raw | AUC_scaled100 | Talked_First | TurnCorr_Lead2 | TurnCorr_Immediate | TurnCorr_Lag2 |
|:---|:---|:---|---:|---:|---:|:---|---:|---:|---:|
| ItsySpider | Maya | emo_anger | 0.001 | 0.783 | 1.630 | Yin | -1 | -1 | -1 |
| ItsySpider | Yin | emo_anger | -0.033 | 0.783 | 1.630 | Yin | -1 | -1 | -1 |
| JackJill | Ana | emo_anger | -0.066 | 3.729 | 4.662 | Franklin | 1 | 1 | 1 |
| JackJill | Franklin | emo_anger | 0.030 | 3.729 | 4.662 | Franklin | 1 | 1 | 1 |
| LittleLamb | Dave | emo_anger | -0.001 | 1.486 | 1.486 | Mary | NA | NA | NA |
| LittleLamb | Mary | emo_anger | -0.031 | 1.486 | 1.486 | Mary | NA | NA | NA |

# Optional: Generate corpus analytics

## `corpus_analytics()`

It is often critical to produce descriptives/summary statistics to
characterize your language sample. This is typically a laborious
process. `corpus_analytics` will do it for you, generating a near
publication ready table of analytics that you can easily export to the
specific journal format of your choice using any number of packages such
as `flextable` or `tinytable`.

### <span style="color: darkred;">Arguments to `corpus_analytics()`:</span>

- `dat_prep` dataframe created by `prep_dyads()`function <br>

``` r
NurseryRhymes_Analytics <-  corpus_analytics(dat_prep=NurseryRhymes_Prepped)
knitr::kable(head(NurseryRhymes_Analytics, 10), format = "simple", digits = 2)
```

| measure                                       |    mean | stdev |    min |    max |
|:----------------------------------------------|--------:|------:|-------:|-------:|
| total number of conversations                 |    3.00 |    NA |     NA |     NA |
| token count all conversations (raw)           | 1506.00 |    NA |     NA |     NA |
| token count all conversations (post-cleaning) | 1032.00 |    NA |     NA |     NA |
| exchange count (by conversation)              |   38.00 | 13.11 |  24.00 |  50.00 |
| word count raw (by conversation)              |  502.00 | 47.03 | 456.00 | 550.00 |
| word count clean (by conversation)            |  344.00 | 48.66 | 312.00 | 400.00 |
| cleaning retention rate (by conversation)     |    0.68 |  0.04 |   0.64 |   0.73 |
| morphemes-per-word (by conversation)          |    1.00 |  0.00 |   1.00 |   1.00 |
| letters-per-word (by conversation)            |    4.22 |  0.14 |   4.12 |   4.38 |
| lexical frequency lg10 (by conversation)      |    3.67 |  0.18 |   3.48 |   3.84 |

<br> <br>

# News and Getting Help

For bugs, feature requests, and general questions, reach out via one of
the following options: <br>
<ul style="list-style-type: none; padding-left: 0;">

<li>

<strong>Bugs/Features:</strong><br>[Open an
Issue](https://github.com/Reilly-ConceptsCognitionLab/ConversationAlign/issues)
</li>

<li>

<strong>Questions:</strong><br>[Join
Discussion](https://github.com/Reilly-ConceptsCognitionLab/SemanticDistance/discussions)
</li>

<li>

<strong>Urgent:</strong><br>Email <code><jamie.reilly@temple.edu></code>
</li>

</ul>

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----message=FALSE, warning=F, echo=F-----------------------------------------
# Load SemanticDistance
library(ConversationAlign)

## ----eval=T, warning=F, message=F---------------------------------------------
NurseryRhymes_Analytics <-  corpus_analytics(dat_prep=NurseryRhymes_Prepped)
knitr::kable(head(NurseryRhymes_Analytics, 15), format = "simple", digits = 2)

## ----message=FALSE, warning=F, echo=F-----------------------------------------
# Load SemanticDistance
library(ConversationAlign)

## ----eval=F, message=F, warning=F---------------------------------------------
# #Example of running the function
# NurseryRhymes_Prepped <- prep_dyads(dat_read=NurseryRhymes, lemmatize=TRUE, omit_stops=T, which_stoplist="Temple_stops25")

## -----------------------------------------------------------------------------
knitr::kable(head(NurseryRhymes_Prepped, 20), format = "simple", digits=2)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----message=FALSE, warning=F, echo=F-----------------------------------------
# Load SemanticDistance
library(ConversationAlign)

## ----eval=F, message=F, warning=F---------------------------------------------
# #will search for folder 'my_transcripts' in your current directory
# MyConvos <- read_dyads()
# 
# #will scan custom folder called 'MyStuff' in your current directory, concatenating all files in that folder into a single dataframe
# MyConvos2 <- read_dyads(my_path='/MyStuff')

## ----eval=F, message=F, warning=F---------------------------------------------
# MaryLittleLamb <- read_1file(MaronGross_2013)
# #print first ten rows of header
# knitr::kable(head(MaronGross_2013, 15), format = "pipe")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----message=FALSE, warning=F, echo=F-----------------------------------------
# Load ConversationAlign
# library(devtools)
# devtools::install_github("https://github.com/Reilly-ConceptsCognitionLab/ConversationAlign.git")
library(ConversationAlign)

## ----eval=T, warning=F, message=F, options(digits=3)--------------------------
MarySumDat <- summarize_dyads(df_prep = NurseryRhymes_Prepped, custom_lags=NULL, sumdat_only = TRUE, corr_type='Pearson') 
colnames(MarySumDat)
knitr::kable(head(MarySumDat, 15), format = "simple", digits = 3)

## ----eval=T, warning=F, message=F---------------------------------------------
MaryShams <- generate_shams(df_prep = NurseryRhymes_Prepped, seed = 10)
MarySumDatShams <- summarize_dyads(df_prep = MaryShams, custom_lags=NULL, sumdat_only = TRUE, corr_type='Pearson') 
knitr::kable(head(MarySumDatShams, 15), format = "simple", digits = 3)

#' generate_shams
#'
#' Generates a permutation of each individual dyad. Shuffled dyads may act as controls to their originals.
#'
#' @param df_prep Output dataframe of prep_dyads().
#' @param seed (Optional) a seed for reproducibility in random sampling
#' @returns
#' A dataframe similar to prepped dyads, with each participant's time series randomly shuffled.
#' @importFrom magrittr %>%
#' @importFrom dplyr group_by
#' @importFrom dplyr summarize
#' @importFrom dplyr across
#' @importFrom dplyr mutate
#' @importFrom dplyr n
#' @export

generate_shams <- function(df_prep, seed = NULL) {
  # if a seed is given, set it
  if (is.null(seed)) { # if not given, pick a random seed
    seed = sample(1:100000, size = 1)
  }

  # summarize down to turn means
  turn_mean_df <- df_prep %>%
    dplyr::group_by(Event_ID, Exchange_Count, Participant_ID) %>%
    dplyr::summarize(
      dplyr::across(
        matches("^(emo_|lex_|phon_|sem_|df_)"),
        ~mean(.x, na.rm = T)
      ),
      # these can be included as a sanity check
      Text_Prep = paste(Text_Prep, collapse = " "),
      Text_Clean = paste(Text_Clean, collapse = " "),
      .groups = "drop"
    )

  # define function that will allow each column to be sampled identically
  sample_seed <- function(x, seed) {
    set.seed(seed)
    return(sample(x, size = length(x), replace = F))
  }

  # shuffle each participant's time series
  sham_df <- turn_mean_df %>%
    dplyr::group_by(Event_ID, Participant_ID) %>%
    dplyr::mutate(
      dplyr::across(
        c(matches("^(emo_|lex_|phon_|sem_|df_)"), Text_Prep, Text_Clean),
        ~sample_seed(.x, seed = seed)
      )
    ) %>%
    dplyr::group_by(Event_ID) %>%
    dplyr::mutate(Turn_Count = 1:dplyr::n(), .after = Event_ID)

  return(sham_df)
}

#' replacements_25
#'
#' String replacement for pattern matching and expanding lots of contractions
#' @name replacements_25
#' @returns
#' nothing, internal function that applies a target list of contractions (e.g., it's) for replacement to prep_dyads
#' @importFrom dplyr mutate
#' @importFrom magrittr %>%
#' @importFrom rlang :=
#' @keywords internal
#' @noRd

replacements_25 <- function(dat, wordcol) {
  # Apply all replacements in sequence
  dat %>%
    # Contractions starting with a/i
    dplyr::mutate({{wordcol}} := gsub("\\baren't\\b", "are not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bcan't\\b", "cannot", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bcould've\\b", "could have", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bcouldn't\\b", "could not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bdidn't\\b", "did not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bdoesn't\\b", "does not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bdon't\\b", "do not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bdunno\\b", "do not know", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bgimme\\b", "give me", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bgonna\\b", "going to", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bgotta\\b", "got to", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bhadn't\\b", "had not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bhasn't\\b", "has not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bhaven't\\b", "have not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bhe'll\\b", "he will", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bhe's\\b", "he is", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bi'd\\b", "i would", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bi'll\\b", "i will", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bi'm\\b", "i am", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bi've\\b", "i have", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bisn't\\b", "is not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bit'll\\b", "it will", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bit's\\b", "it is", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bkinda\\b", "kind of", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\blemme\\b", "let me", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bmight've\\b", "might have", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bmightn't\\b", "might not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bmust've\\b", "must have", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bmustn't\\b", "must not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bneedn't\\b", "need not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bshe'll\\b", "she will", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bshe's\\b", "she is", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bshould've\\b", "should have", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bshouldn't\\b", "should not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bsorta\\b", "sort of", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bthat's\\b", "that is", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bthere'd\\b", "there would", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bthey'll\\b", "they will", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bthey're\\b", "they are", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bthey've\\b", "they have", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bwanna\\b", "want to", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bwasn't\\b", "was not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bwe'll\\b", "we will", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bwe're\\b", "we are", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bwe've\\b", "we have", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bweren't\\b", "were not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bwon't\\b", "will not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bwould've\\b", "would have", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\bwouldn't\\b", "would not", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\byou'd\\b", "you would", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\byou'll\\b", "you will", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\byou're\\b", "you are", {{wordcol}}, ignore.case = TRUE)) %>%
    dplyr::mutate({{wordcol}} := gsub("\\byou've\\b", "you have", {{wordcol}}, ignore.case = TRUE))
}

#' @include utils.R
NULL

#' Package Loading and Data Initialization
#'
#' @description Handles package initialization including loading required datasets
#' from GitHub or local fallback files.
#' @keywords internal
#' @returns
#' nothing, loads data from external repository that will be needed by other package functions
#' @importFrom utils download.file
#' @importFrom tools R_user_dir
#' @noRd
#'
.onLoad <- function(libname, pkgname) {
  # Create package environment
  pkg_env <- asNamespace(pkgname)

  # Critical datasets
  critical_datasets <- c("MIT_stops", "lookup_Jul25", "SMART_stops",
                         "CA_orig_stops", "Temple_stops25")

  #load from GitHub repo
  loaded_from <- tryCatch({
    repo_url <- "https://raw.githubusercontent.com/Reilly-ConceptsCognitionLab/ConversationAlign_Data/main/data/"
    temp_dir <- tempdir()

    for(ds in critical_datasets) {
      temp_file <- file.path(temp_dir, paste0(ds, ".rda"))
      utils::download.file(
        url = paste0(repo_url, ds, ".rda"),
        destfile = temp_file,
        mode = "wb",
        quiet = TRUE
      )
      load(temp_file, envir = pkg_env)
      unlink(temp_file)
    }
    "github"
  }, error = function(e) {
    # Fallback to cache
    cache_dir <- tools::R_user_dir(pkgname, which = "cache")
    cached_files <- file.path(cache_dir, paste0(critical_datasets, ".rda"))

    available <- file.exists(cached_files)
    if(any(available)) {
      for(cf in cached_files[available]) {
        load(cf, envir = pkg_env)
      }
      "cache"
    } else {
      "none"
    }
  })

  # Set package option
  options(ConversationAlign.data_source = loaded_from)
}

.onAttach <- function(libname, pkgname) {
  pkg_env <- asNamespace(pkgname)
  critical_datasets <- c("MIT_stops", "lookup_Jul25", "SMART_stops",
                         "CA_orig_stops", "Temple_stops25")

  still_missing <- setdiff(critical_datasets, ls(envir = pkg_env))

  if(length(still_missing) > 0) {
    loaded_from <- getOption("ConversationAlign.data_source", default = "none")

    msg_type <- if(loaded_from == "none") "error" else "warning"
    msg <- switch(
      msg_type,
      "error" = paste(
        "Critical data missing:", paste(still_missing, collapse = ", "),
        "\nPlease use refresh_data() or contact maintainers"
      ),
      "warning" = paste(
        "Using cached data (missing:", paste(still_missing, collapse = ", "), ")",
        "\nSome features unavailable - try refresh_data()"
      )
    )

    if(msg_type == "error") {
      warning(msg, call. = FALSE, immediate. = TRUE)
    } else {
      packageStartupMessage(msg)
    }
  }
}

#'
#' computes lagged correlations alignment measure across partners within each conversation
#' @name compute_lagcorr_noYRMisc
#' @returns
#' internal function to summarize_dyads that produces a dataframe with lagged correlations across turns (-2,0,2 as default) for each dimension of interest.
#' @importFrom dplyr bind_rows
#' @importFrom dplyr group_by
#' @importFrom dplyr mutate
#' @importFrom dplyr rename_with
#' @importFrom dplyr select
#' @importFrom dplyr summarise
#' @importFrom magrittr %>%
#' @importFrom stats cor.test
#' @importFrom tidyr pivot_wider
#' @importFrom tidyselect contains
#' @importFrom tidyselect ends_with
#' @keywords internal
#' @noRd

compute_lagcorr <- function(df_prep, lags = c(-2, 0, 2), corr_type = "Pearson") {
  # Validate correlation type
  if (!corr_type %in% c("Pearson", "Spearman")) {
    stop("corr_type must be either 'Pearson' or 'Spearman'")
  }
  corr_type <- tolower(corr_type)

  # Select alignment variables
  align_var <- grep("^(emo_|lex_|sem_|phon_)", colnames(df_prep), value = TRUE, ignore.case = TRUE)
  # Remove empty levels of all factors
  df_prep <- droplevels(df_prep)

  # Add participant pair variable
  df_prep <- df_prep %>%
    dplyr::group_by(Event_ID) %>%
    dplyr::mutate(Participant_Pair = paste(sort(unique(Participant_ID)), collapse = "---")) %>%
    dplyr::ungroup()

  # Define lag correlation function
  lag_corr_dyads <- function(df_prep, corr_type) {
    align_var <- colnames(df_prep[, which(colnames(df_prep) %in% align_var)])
    df_list <- split(df_prep, f = df_prep$Event_ID)

    # Iterate over each split data frame
    output_df_list <- lapply(df_list, function(df) {
      # Establish participant names and S1/S2 keys according to who speaks first
      participantvec <- unique(df$Participant_ID)
      names(participantvec) <- c("S1", "S2")
      df$Participant_ID <- gsub(participantvec[1], names(participantvec)[1], df$Participant_ID)
      df$Participant_ID <- gsub(participantvec[2], names(participantvec)[2], df$Participant_ID)

      # Create wide data frame with aggregated scores
      df_wide <- df %>%
        dplyr::group_by(Event_ID, Exchange_Count, Participant_ID, .add = FALSE) %>%
        dplyr::summarise(dplyr::across(tidyselect::contains(align_var), ~ mean(.x, na.rm = TRUE)),
                         .groups = "drop") %>%
        tidyr::pivot_wider(names_from = tidyselect::contains("Participant_ID"),
                           values_from = align_var)

      # Handle single alignment variable case
      if (length(align_var) == 1) {
        colnames(df_wide)[which(colnames(df_wide) %in% c("S1", "S2"))] <-
          paste(align_var[1], colnames(df_wide)[which(colnames(df_wide) %in% c("S1", "S2"))], sep = "_")
      }

      df_wide <- df_wide %>%
        dplyr::select(Event_ID, Exchange_Count, tidyselect::contains(align_var))

      # Remove rows with NA values
      rows_with_na_ind <- apply(df_wide[, which(colnames(df_wide) %in% paste(align_var, "S1", sep = "_") |
                                                  colnames(df_wide) %in% paste(align_var, "S2", sep = "_"))], 1,
                                function(x) any(is.na(x) == TRUE & is.nan(x) == FALSE))
      if (any(rows_with_na_ind)) {
        df_wide <- df_wide[!rows_with_na_ind, ]
      }

      # Interpolate missing values
      interp_df <- df_wide %>%
        dplyr::mutate(dplyr::across(tidyselect::contains(align_var),
                                    ~ zoo::na.approx(.x, na.rm = FALSE))) %>%
        tidyr::fill(names(df_wide[, which(colnames(df_wide) %in% paste(align_var, "S1", sep = "_") |
                                            colnames(df_wide) %in% paste(align_var, "S2", sep = "_"))]),
                    .direction = "updown")

      # Prepare x and y variables
      x_vars <- interp_df %>%
        dplyr::select(c('Event_ID') | c(tidyselect::ends_with("_S1"))) %>%
        dplyr::rename_with(~stringr::str_replace(., "_S1", ""),
                           tidyselect::ends_with("_S1"))
      y_vars <- interp_df %>%
        dplyr::select(c('Event_ID') | c(tidyselect::ends_with("_S2"))) %>%
        dplyr::rename_with(~stringr::str_replace(., "_S2", ""),
                           tidyselect::ends_with("_S2"))

      # Modified function to calculate lagged correlation (NO YRmisc dependency)
      calculate_lag_corr <- function(x_vars, y_vars, dim, align_var, corr_type) {
        dim_x_vars <- x_vars[[dim]]
        dim_y_vars <- y_vars[[dim]]

        # For Spearman, convert to ranks
        if (corr_type == "Spearman") {
          dim_x_vars <- rank(dim_x_vars, na.last = "keep")
          dim_y_vars <- rank(dim_y_vars, na.last = "keep")
        }

        # Directly compute correlations for each requested lag
        cor_vals <- sapply(lags, function(L) {
          n <- length(dim_x_vars)
          if (L == 0) {
            cor(dim_x_vars, dim_y_vars, use = "complete.obs", method = corr_type)
          } else if (L < 0) {
            L <- abs(L)
            # y lags behind x: correlate x[t] with y[t-L]  (L>0)
            if (L >= n) return(NA)
            x_sel <- dim_x_vars[(L+1):n]
            y_sel <- dim_y_vars[1:(n-L)]
            cor(x_sel, y_sel, use = "complete.obs", method = corr_type)
          } else {  # L < 0
            # y leads x: correlate x[t] with y[t+|L|]
            #k <- abs(L)
            k <- L
            if (k >= n) return(NA)
            x_sel <- dim_x_vars[1:(n-k)]
            y_sel <- dim_y_vars[(k+1):n]
            cor(x_sel, y_sel, use = "complete.obs", method = corr_type)
          }
        })

        # Create column names as in the original output
        selectNames <- sapply(lags, function(x) {
          if(x < 0) paste0("TurnCorr_Lead", abs(x))
          else if(x > 0) paste0("TurnCorr_Lag", x)
          else "TurnCorr_Immediate"
        })

        # Build a one-row data frame with the selected names
        lo_full <- as.data.frame(as.list(setNames(cor_vals, selectNames)),
                                 stringsAsFactors = FALSE)
        lo_full %>% dplyr::select(tidyselect::all_of(selectNames))
      }

      # Calculate correlations for each dimension
      dyad_dim_list <- lapply(align_var, function(dim) {
        lag_results_df <- calculate_lag_corr(x_vars, y_vars, dim, align_var, corr_type)
        colnames(lag_results_df) <- paste(colnames(lag_results_df), dim, sep = "-")
        if (match(dim, align_var) == 1) {
          lag_results_df$Participant_ID <- paste(participantvec, collapse = "---")
        }
        lag_results_df
      })

      # Combine results
      dyad_covar <- dplyr::bind_cols(dyad_dim_list, Event_ID = unique(df$Event_ID))
      dyad_covar$Talked_First <- participantvec[1]
      dyad_covar
    })

    dplyr::bind_rows(output_df_list)
  }

  # Compute and process correlations
  covar_df <- lag_corr_dyads(df_prep = df_prep, corr_type = corr_type)

  covar_df <- covar_df %>% dplyr::select(!c(Participant_ID)) %>%
    tidyr::pivot_longer(cols = tidyselect::contains("TurnCorr"),
                        names_pattern = "(.*)-(.*)",
                        names_to = c(".value", "Dimension"))

  row.names(covar_df) <- NULL
  return(covar_df)
}

# unit testing for computing correlations

# prepare data for lagged corr testing
# should give corr of 1 when lagged by 2
input <- data.frame(
  Event_ID = "corr_test",
  Participant_ID = rep(c("P1", "P2"), 50),
  Exchange_Count = rep(1:50, each = 2),
  emo_anger = c(rbind(1:50, c(0, 0, 1:48))) # interleave two sequences, one lagged by 2
)

test_that("computed pearson correlation is correct", {
  # lagged by two should be equal to 1
  p_output <- compute_lagcorr(input, corr_type = "Pearson")
  expect_equal(p_output$TurnCorr_Lag2[1], 1)

})


test_that("computed spearman correlation is correct", {
  # lagged by two should be equal to 1
  p_output <- compute_lagcorr(input, corr_type = "Spearman")
  expect_equal(p_output$TurnCorr_Lag2[1], 1)
})

#' prep_dyads
#'
#' Cleans, vectorizes and appends lexical norms to all content words in a language corpus.
#' User guides options for stopword removal and lemmatization. User selects up to three psycholinguistic dimensions to yoke norms
#' on each content word in the original conversation transcript.
#' @name prep_dyads
#' @param dat_read dataframe produced from read_dyads() function
#' @param omit_stops option to remove stopwords, default TRUE
#' @param lemmatize logical, should words be lemmatized (switched to base morphological form), default is TRUE
#' @param which_stoplist user-specified stopword removal method with options including "none", "SMART", "MIT_stops", "CA_OriginalStops", or "Temple_Stopwords25".
#' "Temple_Stopwords25 is the default list
#' @param remove_backchannel logical, should turns that are full of stopwords (e.g., "Uhm yeah") be preserved as NAs or removed. Removal will 'squish' the turn before and after together into one. If NAs are preserved they are later interpolated.
#' @param verbose display detailed output such as error messages and progress (default is TRUE)
#' @returns
#' dataframe with text cleaned and vectorized to a one word per-row format.
#' Lexical norms and metadata are appended to each content word. Cleaned text appears under a new column
#' called 'Text_Clean'. Any selected dimensions (e.g., word length) and metadata are also appended to each word along
#' with speaker identity, turn, and Event_ID (conversation identifier).
#' @importFrom dplyr select
#' @importFrom dplyr group_by
#' @importFrom dplyr mutate
#' @importFrom dplyr na_if
#' @importFrom dplyr ungroup
#' @importFrom dplyr consecutive_id
#' @importFrom magrittr %>%
#' @importFrom stringi stri_replace_all_fixed
#' @importFrom stringi stri_replace_all_regex
#' @importFrom stringr str_trim
#' @importFrom stringr str_squish
#' @importFrom stringr str_replace_all
#' @importFrom textstem lemmatize_strings
#' @importFrom tidyr separate_rows
#' @importFrom tidyselect all_of
#' @importFrom tidyselect any_of
#' @importFrom tidyselect contains
#' @importFrom utils select.list
#' @export

prep_dyads <- function(dat_read, lemmatize = TRUE, omit_stops = TRUE,
                       which_stoplist = "Temple_stops25",
                       remove_backchannel = FALSE, verbose = TRUE) {
  # Verification steps
  if (nrow(dat_read) == 0) {
    stop("Input dataframe is empty.")
  }
  if (!"Text_Raw" %in% names(dat_read)) {
    stop("Column 'Text_Raw' not found.")
  }
  if (!"Participant_ID" %in% names(dat_read)) {
    stop("Column 'Participant_ID' not found.")
  }
  # verify backchanneling argument
  if (!is.logical(remove_backchannel)) {
    stop("Argument 'keep_backchannelling' must be logical")
  }

  # Define stopword lists
  stopwords_lists <- list(MIT_stops = MIT_stops,
                          Temple_stops25 = Temple_stops25,
                          SMART_stops = SMART_stops,
                          CA_orig_stops = CA_orig_stops)

  # Validate stoplist selection
  if (omit_stops && !which_stoplist %in% names(stopwords_lists)) {
    stop("Invalid stoplist specified. Choose from: ", paste(names(stopwords_lists), collapse = ", "))
  }

  # Only prompt for stoplist if omit_stops is TRUE and which_stoplist is NULL
  if (omit_stops && is.null(which_stoplist)) {
    if (verbose) {
      message("Available stopword lists:")
      message("1. Temple_stops25\n2. MIT_stops\n3. SMART_stops\n4. CA_orig_stops")
    }
    choice <- readline(prompt = "Enter number of stoplist to use (1-4): ")
    which_stoplist <- c("Temple_stops25", "MIT_stops", "SMART_stops", "CA_orig_stops")[as.integer(choice)]
  }

  # create Turn_Count variable
  dat_prep <- dat_read %>% group_by(Event_ID) %>%
    mutate(switch_mark = Participant_ID != dplyr::lag(Participant_ID, default = first(Participant_ID)),
           Turn_Count = cumsum(switch_mark) + 1) %>% select(-switch_mark) %>% ungroup()

  # Text Processing Pipeline
  dat_prep <- dat_prep %>% mutate(Participant_ID = as.factor(Participant_ID),
                                  Event_ID = as.factor(Event_ID), Text_Prep = tolower(Text_Raw)) %>% select(-Text_Raw)

  # Standardize apostrophes
  dat_prep <- dat_prep %>%
    mutate(Text_Prep = stringi::stri_replace_all_regex(Text_Prep, "[\u2018\u2019\u02BC\u201B\uFF07\u0092\u0091\u0060\u00B4\u2032\u2035]", "'"))

  # Remove non-alphabetic characters except apostrophes
  dat_prep <- dat_prep %>% dplyr::mutate(Text_Prep = stringi::stri_replace_all_regex(Text_Prep, "[^a-zA-Z']", " "))

  # Clean whitespace
  dat_prep <- dat_prep %>% dplyr::mutate(Text_Prep = str_squish(gsub("\\s+", " ", Text_Prep)))

  # Split into words
  dat_prep <- dat_prep %>% tidyr::separate_rows(Text_Prep, sep = "[[:space:]]+")

  # Clean text
  dat_prep <- dat_prep %>% mutate(Text_Prep = stringi::stri_replace_all_regex(Text_Prep, "[^a-z']", ""))

  # ASCII conversion
  dat_prep <- dat_prep %>% mutate(Text_Prep = iconv(Text_Prep, to = "ASCII//TRANSLIT", sub = ""),
                                  Text_Prep = stringi::stri_replace_all_regex(Text_Prep, "[^[:alnum:]']", ""))

  # Apply contractions replacement do NOT quote the column name
  dat_prep <- replacements_25(dat = dat_prep, wordcol = Text_Prep)

  # Split again after contractions
  dat_prep <- dat_prep %>% tidyr::separate_rows(Text_Prep, sep = "[[:space:]]+")

  # Final processing and lemmatization
  df_prep <- dat_prep %>% mutate(Text_Clean = ifelse(stringi::stri_isempty(Text_Prep), NA, Text_Prep),
                                 Text_Clean = if(lemmatize) textstem::lemmatize_strings(Text_Clean) else Text_Clean)

  # Stopword removal
  if (omit_stops) {
    stopwords <- stopwords_lists[[which_stoplist]]$word
    df_prep <- df_prep %>% mutate(Text_Clean = ifelse(Text_Clean %in% stopwords, NA, Text_Clean))
  }

  # Variable selection
  possible_vars <- setdiff(colnames(lookup_Jul25), "word")

  repeat {
    if (verbose) {
      message("Available variables:")
      for (i in seq_along(possible_vars)) {
        message(sprintf("%d. %s", i, possible_vars[i]))
      }
      message("\nSelect up to 3 variables you want to analyze alignment on.")
      message("Enter the numbers separated by spaces (e.g., 1 3 5) then hit enter: ")
    }

    input <- readline()
    selected_indices <- suppressWarnings(as.numeric(unlist(strsplit(trimws(input), "\\s+"))))

    if (length(selected_indices) == 0 || any(is.na(selected_indices))) {
      warning("Invalid input. Please enter numbers only, separated by spaces.")
      next
    }
    if (any(selected_indices < 1 | selected_indices > length(possible_vars))) {
      warning("Invalid selection. Please enter numbers between 1 and ", length(possible_vars))
      next
    }
    if (length(selected_indices) > 3) {
      warning("You selected more than 3 variables. Please select 3 or fewer.")
      next
    }

    myvars <- possible_vars[selected_indices]
    break
  }

  # [Rest of the function remains unchanged]
  var_selected <- lookup_Jul25 %>% dplyr::select(word, tidyselect::all_of(myvars))

  # Join with psycholinguistic measures
  df_prep <- df_prep %>% left_join(var_selected, by = c("Text_Clean" = "word")) %>%
    mutate(Exchange_Count = ceiling(Turn_Count / 2))

  # Reorder columns
  df_prep <- df_prep %>% select(Event_ID, Participant_ID, Exchange_Count, Turn_Count,
                                Text_Prep, Text_Clean, all_of(myvars), everything())

  # # if backchanneling is set to be removed, squish turns together
  if (remove_backchannel == TRUE) {
    df_prep <- df_prep[!is.na(df_prep$Text_Clean), ]
    df_prep <- df_prep %>%
      dplyr::group_by(Event_ID) %>%
      dplyr::mutate(
        Turn_Count = dplyr::consecutive_id(Participant_ID),
        Exchange_Count = ceiling(Turn_Count / 2)
      ) %>%
      dplyr::ungroup()
  }

  return(df_prep)
}

#' Load all .rda files from a GitHub data folder into the package environment
#'
#' @param repo GitHub repository (e.g., "username/repo")
#' @param branch Branch name (default: "main")
#' @param data_folder Remote folder containing .rda files (default: "data/")
#' @param envir Environment to load into (default: package namespace)
#' @returns
#' nothing, loads data (as rda files) from github repository needed for other package functions
#' @importFrom httr GET

load_github_data <- function(
    repo = "Reilly-ConceptsCognitionLab/ConversationAlign_Data",
    branch = "main",
    data_folder = "data",
    envir = parent.frame()
) {
  # Validate inputs
  if (missing(repo)) stop("Argument 'repo' is required")

  # Construct API URL
  api_url <- sprintf(
    "https://api.github.com/repos/%s/contents/%s?ref=%s",
    repo, data_folder, branch
  )

  # Add GitHub PAT if available
  headers <- if (nzchar(Sys.getenv("GITHUB_PAT"))) {
    httr::add_headers(Authorization = paste("token", Sys.getenv("GITHUB_PAT")))
  } else {
    list()
  }

  # Make API request
  response <- tryCatch({
    httr::GET(api_url, headers)
  }, error = function(e) {
    stop("GitHub API request failed. Error: ", e$message)
  })

  if (httr::http_error(response)) {
    stop(sprintf("GitHub API error [%s]: %s",
                 httr::status_code(response),
                 httr::content(response, "text")))
  }

  # Process response
  files <- httr::content(response)
  rda_files <- grep("\\.rda$", sapply(files, `[[`, "name"),
                    value = TRUE, ignore.case = TRUE)

  # Download and load each file
  for (file in rda_files) {
    raw_url <- sprintf("https://raw.githubusercontent.com/%s/%s/%s/%s",
                       repo, branch, data_folder, file)
    temp_file <- tempfile(fileext = ".rda")
    utils::download.file(raw_url, temp_file, mode = "wb", quiet = TRUE)
    load(temp_file, envir = envir)
    unlink(temp_file)
  }
}

#' corpus_analytics
#'
#' Produces a table of corpus analytics including numbers of complete observations at each step, word counts, lexical diversity (e.g., TTR), stopword ratios, etc. Granularity of the summary statistics are guided by the user (e.g., by conversation, by conversation and speaker, collapsed all)
#' @name corpus_analytics
#' @param dat_prep takes dataframe produced from the df_prep() function
#' @returns
#' dataframe with summary statistics (mean, SD, range) for numerous corpus analytics (e.g., token count, type-token-ratio, word-count-per-turn) for the target conversation corpus. Summary data structured in table format for easy export to a journal method section.
#' @importFrom dplyr across
#' @importFrom dplyr bind_rows
#' @importFrom dplyr everything
#' @importFrom dplyr filter
#' @importFrom dplyr group_by
#' @importFrom dplyr left_join
#' @importFrom dplyr matches
#' @importFrom dplyr mutate
#' @importFrom dplyr n_distinct
#' @importFrom dplyr rename
#' @importFrom dplyr select
#' @importFrom dplyr summarize
#' @importFrom dplyr ungroup
#' @importFrom magrittr %>%
#' @importFrom purrr map_dfr
#' @importFrom stats sd
#' @importFrom stats na.omit
#' @importFrom stringr str_subset
#' @importFrom stringr str_trim
#' @importFrom tibble tibble
#' @importFrom tidyr pivot_longer
#' @importFrom tidyr pivot_wider
#' @importFrom tidyr separate
#' @importFrom tidyselect where
#' @export

#Text_Prep (just with contractions split), Text_Clean (no stopwords), Any dimension in dataframe counts (emo_, phono_, lex_, sem_)
#Group_by Event_ID for most stats
# WordCount, TTR, Morphemes_per_word, Letters_per_word, Exchanges, NSyll, Words_Per_Turn

#Complete this summarize function so that the grouped dataframe outputs new variables:
# 'conversations (total)' = number of distinct levels of 'Event_ID'
# 'tokens (all conversations) = sum of all complete obs of Text_Prep across all levels of Event_ID
# 'exchanges-per-conversation (mean, sd, min, max) = exchanges per 'Event_ID' derived by the maximum exchange_count for each Event_ID.
# words-per-conversation_raw (mean, sd, min, max): complete observations of 'Text_Prep' for each level of Event_ID
# words-per-conversation_clean (mean, sd, min, max): complete observations of 'Text_Clean' for each level of Event_ID
# retention rate post cleaning = Number of complete observations for: Text_Clean / Text_Prep
# morphemes-per-word (mean, sd, min, max): across all complete obs of 'lex_n_morphemes' by level of Event_ID
# letter count (mean, sd, min, max): across all complete obs of 'phon_n_letters' by level of Event_ID
# word freq (mean, sd, min, max): across all complete obs of 'lex_freqlg10' by level of Event_ID
# words-per-turn (raw) (mean, sd, min, max): group_by turn_count, Event_ID, count complete obs per level of turn_count in Text_Prep
# words-per-turn (clean) (mean, sd, min, max): group_by turn_count, Event_ID, count complete obs per level of turn_count in Text_Clean
# TTR (raw): Group by Event_ID, distinct Text_Prep divided by Text_Prep
# TTR (clean): Group by Event_ID, distinct Text_Clean divided by Text_Clean

corpus_analytics <- function(dat_prep) {
  # Select and prepare data
  dat_prep <- dat_prep %>%
    dplyr::select(Event_ID, Participant_ID, Exchange_Count, Turn_Count, Text_Prep, Text_Clean,
                  dplyr::matches("^emo_|^phon_|^sem_|^lex_"))

  lookup <- lookup_Jul25 %>% dplyr::select(word, phon_n_lett, phon_nsyll, lex_freqlg10, lex_n_morphemes)

  # Join dat_prep with psycholing vars norms for table (NO ROUNDING)
  dat_prep_plusvals <- dat_prep %>%
    dplyr::left_join(lookup, by = c("Text_Clean" = "word"))
  # Removed: mutate(across(where(is.numeric), ~round(., 2)))

  # Calculate totals counts (n-conversations, n-tokens ALL)
  total_tokens_raw <- sum(!is.na(dat_prep_plusvals$Text_Prep))
  total_tokens_clean <- sum(!is.na(dat_prep_plusvals$Text_Clean))
  n_conversations <- n_distinct(dat_prep_plusvals$Event_ID)

  # First calculate words per turn for each turn in each conversation
  words_per_turn_stats <- dat_prep_plusvals %>%
    group_by(Event_ID, Turn_Count) %>%
    summarize(
      words_per_turn_raw = sum(!is.na(Text_Prep)),
      words_per_turn_clean = sum(!is.na(Text_Clean)),
      .groups = 'drop'
    ) %>%
    group_by(Event_ID) %>%
    summarize(
      words_per_turn_raw = mean(words_per_turn_raw, na.rm = TRUE),
      words_per_turn_clean = mean(words_per_turn_clean, na.rm = TRUE),
      .groups = 'drop'
    )

  # Stats for each conversation (NO ROUNDING)
  conversation_stats <- dat_prep_plusvals %>%
    group_by(Event_ID) %>%
    summarize(
      total_exchanges = max(Exchange_Count, na.rm = TRUE),
      words_raw = sum(!is.na(Text_Prep)),
      words_clean = sum(!is.na(Text_Clean)),
      retention_rate = sum(!is.na(Text_Clean))/sum(!is.na(Text_Prep)),
      morphemes = mean(lex_n_morphemes, na.rm = TRUE),
      letters = mean(phon_n_lett, na.rm = TRUE),
      freq = mean(lex_freqlg10, na.rm = TRUE),
      ttr_raw = dplyr::n_distinct(Text_Prep, na.rm = TRUE)/sum(!is.na(Text_Prep)),
      ttr_clean = dplyr::n_distinct(Text_Clean, na.rm = TRUE)/sum(!is.na(Text_Clean)),
      .groups = 'drop'
    ) %>%
    left_join(words_per_turn_stats, by = "Event_ID")
  # Removed: mutate(across(where(is.numeric), ~round(., 2)))

  # Verify we have multiple conversations to compute SD
  if (nrow(conversation_stats) < 2) {
    warning("Insufficient conversations (n < 2) to compute meaningful standard deviations")
  }

  # Calculate summary statistics across all conversations (NO ROUNDING)
  result <- purrr::map_dfr(
    list(
      "exchange count (by conversation)" = conversation_stats$total_exchanges,
      "word count raw (by conversation)" = conversation_stats$words_raw,
      "word count clean (by conversation)" = conversation_stats$words_clean,
      "cleaning retention rate (by conversation)" = conversation_stats$retention_rate,
      "morphemes-per-word (by conversation)" = conversation_stats$morphemes,
      "letters-per-word (by conversation)" = conversation_stats$letters,
      "lexical frequency lg10 (by conversation)" = conversation_stats$freq,
      "words-per-turn raw (by conversation)" = conversation_stats$words_per_turn_raw,
      "words-per-turn clean (by conversation)" = conversation_stats$words_per_turn_clean,
      "TTR raw (by conversation)" = conversation_stats$ttr_raw,
      "TTR clean (by conversation)" = conversation_stats$ttr_clean
    ),
    function(x) {
      tibble(
        mean = mean(x, na.rm = TRUE),
        stdev = ifelse(length(na.omit(x)) > 1, sd(x, na.rm = TRUE), NA_real_),
        min = min(x, na.rm = TRUE),
        max = max(x, na.rm = TRUE)
      )
    },
    .id = "measure"
  )
  # Removed: mutate(across(c(mean, stdev, min, max), ~round(., 2)))

  # Add summary rows for corpus-level totals
  summary_rows <- tibble(
    measure = c("total number of conversations", "token count all conversations (raw)", "token count all conversations (post-cleaning)"),
    mean = c(n_conversations, total_tokens_raw, total_tokens_clean),
    stdev = NA_real_,
    min = NA_real_,
    max = NA_real_
  )

  # Combine results
  final_result <- bind_rows(summary_rows, result) %>%
    select(measure, mean, stdev, min, max)

  return(final_result)
}

#' read_dyads
#'
#' Reads pre-formatted dyadic (2 interlocutor) conversation transcripts from your machine. Transcripts must be either csv or txt format. IF you are supplying a txt file, your transcript must be formatted as an otter.ai txt file export. Your options for using csv files are more flexible. ConversationAlign minimally requires a csv file with two columns, denoting interlocutor and text. Each separate conversation transcript should be saved as a separate file. ConversationAlign will use the file names as a document ID. Within the read dyads function, set the my_path argument as the directory path to the local folder containing your transcripts on your machine (e.g., "my_transcripts"). Please see our github page for examples of properly formatted transcripts: https://github.com/Reilly-ConceptsCognitionLab/ConversationAlign
#'
#' @name read_dyads
#' @param my_path folder of conversation transcripts in csv or txt format
#' @returns
#' a dataframe where each individual conversation transcript in a user's directory has been concatenated.
#' read_dyads appends a unique document identifier to each conversation transcript appending its unique filename as a factor level to 'Event_ID'.
#' @importFrom magrittr %>%
#' @importFrom dplyr bind_rows
#' @importFrom utils read.csv
#' @export
read_dyads <- function(my_path = "my_transcripts") {
  read_otter_transcript <- function(file_path) {
    lines <- readLines(file_path) #read otter ai file
    #removes otter ai watermark if it is present
    if (any(grepl("otter.ai", lines)) == TRUE) {
      lines <- as.character(lines[-(grep("otter.ai", lines))])}
    num_lines <- length(lines) #create a var for number of lines
    speaker <- character()
    time <- character()
    text <- character()

    #process lines of dialogue
    current_line <- 1
    while (current_line <= num_lines) {

      first_line_vec <- strsplit(lines[current_line], " ")
      speaker_time <- first_line_vec[[1]]
      speaker <- c(speaker, speaker_time[1]) #select speaker
      # check if time is included in mm:ss format and if so add it to the time column
      if (any(grepl(":", speaker_time)) == TRUE) {
        timeadd <- grep(":", speaker_time)
      }
      else {timeadd <- NA}

      time <- append(time, timeadd)
      #select lines of speech
      speech_lines <- character()
      line_counter <- current_line + 1
      while (line_counter <= num_lines && lines[line_counter] != "") { #if not max line/empty line
        speech_lines <- c(speech_lines, lines[line_counter]) #add text on line to speech text vector
        line_counter <- line_counter + 1
      }
      text <- append(text, paste(speech_lines, collapse = " "))
      current_line <- line_counter + 1 #move to next speaker
    }
    #create df
    transcript_df <- data.frame(Participant_ID = speaker,
                                Time = time,
                                Text_Raw = text,
                                stringsAsFactors = FALSE)
    return(transcript_df)
  }
  #END DEFINE OTTER READ TRANSCRIPT .TXT FILE FUNCTION

  #DEFINE READ ME TXT FILE FUNCTION
  read_dyads_txt <- function(my_path){
    if (any(grepl("*.txt$", list.files(path = my_path, pattern = ".", full.names = TRUE, recursive = TRUE))) == TRUE) {
      file_list_txt <- list.files(path = my_path, pattern = "*.txt$", full.names = TRUE, recursive = TRUE) #list files with .txt ending
      file_names_txt <- list.files(path = my_path, pattern = ".txt$", full.names = FALSE, recursive = TRUE)
      file_names_txt <- gsub('.*/ ?(\\w+)', '\\1', file_names_txt)
      file_names_txt <- gsub(".txt$", "", file_names_txt)

      txtdata <- lapply(file_list_txt, function(x) {
        #runs txt files names through otter reading function
        xorf <- read_otter_transcript(x)
        #selects those that were properly transcribed by otter, still need to add other parameters
        if (ncol(xorf) == 3) {
          x <- xorf
        }
        else {
          # Other function for reading in non-otter txt transcripts can go here later
          stop(paste("Unable to read transcript ", as.character(match(x, file_list_txt)), " as an otter transcript. Please refer to the ConversationAlign GitHub page for examples of properly formatted transcripts.", sep = ""), call. = FALSE)
        }
      })

      #adds a doc id column to each transcript based on its name attribute
      txtdata <- lapply(file_names_txt, function(x){
        txtdata[[match(x, file_names_txt)]] <- cbind(Event_ID = rep(x, nrow(txtdata[[match(x, file_names_txt)]])), txtdata[[match(x, file_names_txt)]])})
      return(txtdata)
    }}
  #END DEFINE READ ME TXT FILE FUNCITON

  #DEFINE READ ME CSV FILE FUNCTION
  read_dyads_csv <- function(my_path) {
    if (any(grepl("*.csv$", list.files(path = my_path, pattern = ".",
                                       full.names = TRUE, recursive = TRUE))) == TRUE) {
      file_list_csv <- list.files(path = my_path, pattern = "*.csv$",
                                  full.names = TRUE, recursive = TRUE)
      file_names_csv <- list.files(path = my_path, pattern = ".csv$",
                                   full.names = FALSE, recursive = TRUE)
      file_names_csv<- gsub('.*/ ?(\\w+)', '\\1', file_names_csv)
      file_names_csv <- gsub(".csv$", "", file_names_csv)

      #creates a list of read in csv data frames
      csvdata <- lapply(file_list_csv, function(x){
        x_read_csv <- read.csv(x, header = TRUE)
        #identify and remove all columns that are entirely NA or empty strings
        sum_nas_es <- apply(x_read_csv, 2, function(y){sum(is.na(y) | y == "")})
        x_read_csv <- x_read_csv[sum_nas_es < nrow(x_read_csv)]
        x_read_csv <- data.frame(x_read_csv)

        #check that the column names are correct
        if ((any(grepl("^speaker$", colnames(x_read_csv), ignore.case = T)) == TRUE |
             any(grepl("^speaker_names_raw$", colnames(x_read_csv), ignore.case = T)) == TRUE |
             any(grepl("^Participant$", colnames(x_read_csv), ignore.case = T)) == TRUE |
             any(grepl("^Interlocutor$", colnames(x_read_csv), ignore.case = T)) == TRUE |
             any(grepl("^Patient$", colnames(x_read_csv), ignore.case = T)) == TRUE |
             any(grepl("^Person$", colnames(x_read_csv), ignore.case = T)) == TRUE |
             any(grepl("^Partner$", colnames(x_read_csv), ignore.case = T)) == TRUE |
             any(grepl("^Source$", colnames(x_read_csv), ignore.case = T)) == TRUE |
             any(grepl("^PID$", colnames(x_read_csv), ignore.case = T)) == TRUE) &
            (any(grepl("^Text$", colnames(x_read_csv), ignore.case = T)) == TRUE |
             any(grepl("^Turn$", colnames(x_read_csv), ignore.case = T)) == TRUE |
             any(grepl("^talker$", colnames(x_read_csv), ignore.case = T)) == TRUE |
             any(grepl("^MyText$", colnames(x_read_csv), ignore.case = T)) == TRUE |
             any(grepl("^Utterance$", colnames(x_read_csv), ignore.case = T)) == TRUE)) {

          #correct the speaker and text names to our conventions
          colnames(x_read_csv)[which(grepl("speaker", colnames(x_read_csv), ignore.case = T) |
                                       grepl("PID", colnames(x_read_csv), ignore.case = T) |
                                       grepl("interlocutor", colnames(x_read_csv), ignore.case = T) |
                                       grepl("patient", colnames(x_read_csv), ignore.case = T) |
                                       grepl("person", colnames(x_read_csv), ignore.case = T) |
                                       grepl("partner", colnames(x_read_csv), ignore.case = T) |
                                       grepl("source", colnames(x_read_csv), ignore.case = T) |
                                       grepl("talker", colnames(x_read_csv), ignore.case = T) |
                                       grepl("participant", colnames(x_read_csv), ignore.case = T))] <- "Participant_ID"

          colnames(x_read_csv)[which(grepl("Text", colnames(x_read_csv), ignore.case = T) |
                                       grepl("utterance", colnames(x_read_csv), ignore.case = T))] <- "Text_Raw"

          x_read_csv <- data.frame(x_read_csv)
          x_final <- x_read_csv
        }

        col_check <- sum(colnames(x_read_csv) %in% c("Participant_ID", "Text_Raw"))

        if (col_check != 2) { #if there are less than two columns
          stop(paste("Function is unable to process csv transcript ", #error stating missing column
                     as.character(match(x, file_list_csv)), #also states the transcript
                     " correctly. Make sure that each transcript includes a column marking who is producing text in each row.
                     When you are preparing your language transcripts, please make sure this column is named one of the following: 'interlocutor', 'person', 'partner', 'source', 'speaker', 'participant', 'PID', or 'speaker_names_raw'.
                     Please make sure your a column containing raw language transcriptions is named 'utterance', 'turn', or 'text'.", sep = ""), call. = FALSE)

        }
        x_final
      })
      csvdata <- lapply(csvdata, data.frame)
      csvdata <- lapply(file_names_csv, function(x){
        csvdata[[match(x, file_names_csv)]] <- cbind(Event_ID = rep(x, nrow(csvdata[[match(x, file_names_csv)]])), csvdata[[match(x, file_names_csv)]])})
      csvdata
    }}
  #END DEFINE READ ME CSV FILE FUNCTION
  #calls two functions to read in txt and csv file transcripts, returning a list.
  txtlist <- read_dyads_txt(my_path)
  csvlist <- read_dyads_csv(my_path)
  all_list <- append(txtlist, csvlist) #append the two lists into one list

  #throws an error if no files are found
  if (length(all_list) == 0) {
    stop("No files found. Please make sure you are providing the local or absolute file path to the desired folder as a character vector. At least one .csv or .txt file must be present.")
  }

  alldf <- dplyr::bind_rows(all_list) #binds the rows of each list into one data frame
  alldf$Event_ID <- as.factor(alldf$Event_ID)
  alldf$Participant_ID <- as.factor(alldf$Participant_ID)

  dat_read <- alldf

  # NEW VALIDATION STEP: Check for exactly two participants per event
  participant_check <-  dat_read  %>%
    dplyr::group_by(Event_ID) %>%
    dplyr::summarize(
      n_participants = n_distinct(Participant_ID),
      .groups = "drop"
    )

  # Identify events that don't have exactly 2 participants
  invalid_events <- participant_check %>%
    dplyr::filter(n_participants != 2) %>%
    dplyr::pull(Event_ID)

  if (length(invalid_events) > 0) {
    error_message <- paste(
      "Check your data! One or more conversations has more or less than two conversation partners.\n",
      "ConversationAlign only works on dyadic transcripts. Each conversation transcript must have ONLY two participants.\n",
      "Problematic Event_ID(s):", paste(invalid_events, collapse = ", ")
    )
    stop(error_message, call. = FALSE)
  }

  return(dat_read)
}

#' read_1file
#'
#' Reads pre-formatted dyadic (2 interlocutor) conversation transcript already imported into your R environment.
#'
#' @name read_1file
#' @param my_dat one conversation transcript already in the R environment
#' @returns
#' a dataframe formatted with 'Event_ID', "Participant_ID", "Text_Raw" fields -- ready for clean_dyads()
#' @export

read_1file <- function(my_dat) {
  #returns name not contents of mydat
  object_name <- deparse(substitute(my_dat))

  # Convert to data frame if not already
  if (!is.data.frame(my_dat)) {
    my_dat <- as.data.frame(my_dat)
  }

  # Store original column names for reference
  original_cols <- colnames(my_dat)

  # Standardize column names (case-insensitive)
  colnames(my_dat) <- tolower(colnames(my_dat))

  # Initialize standardized columns
  standardized_cols <- colnames(my_dat)

  # Participant ID detection and standardization
  participant_pattern <- "speaker|speaker_names_raw|participant|interlocutor|patient|person|partner|source|pid|talker"
  participant_idx <- grepl(participant_pattern, colnames(my_dat))
  if (sum(participant_idx) > 0) {
    standardized_cols[participant_idx] <- "Participant_ID"
  }

  # RawText detection and standardization
  text_pattern <- "text|turn|talker|mytext|utterance|my_text"
  text_idx <- grepl(text_pattern, colnames(my_dat))
  if (sum(text_idx) > 0) {
    standardized_cols[text_idx] <- "RawText"
  }

  # Apply standardized names
  colnames(my_dat) <- standardized_cols

  # Check required columns exist
  required_cols <- c("Participant_ID", "RawText")
  missing_cols <- setdiff(required_cols, colnames(my_dat))

  if (length(missing_cols) > 0) {
    stop(paste("Missing required columns:",
               paste(missing_cols, collapse = ", "),
               "\nAvailable columns:",
               paste(original_cols, collapse = ", "),
               "\nExpected participant columns should match:", participant_pattern,
               "\nExpected text columns should match:", text_pattern),
         call. = FALSE)
  }

  # Add Event_ID using the object's name
  my_dat$Event_ID <- object_name

  # Convert ID columns to factors
  id_cols <- c("Event_ID", "Participant_ID")
  for (col in id_cols) {
    if (col %in% colnames(my_dat)) {
      my_dat[[col]] <- as.factor(my_dat[[col]])
    }
  }

  # Reorder columns to put standard ones first
  standard_cols <- c("Event_ID", "Participant_ID", "Text_Raw")
  other_cols <- setdiff(colnames(my_dat), standard_cols)
  my_dat <- my_dat[, c(standard_cols, other_cols)]

  return(my_dat)
}

#' summarize_dyads
#'
#' Calculates and appends 3 measures for quantifying alignment. Appends the averaged value for each selected dimension by turn and speaker. Calculates and Spearman's rank correlation between interlocutor time series and appends by transcript. Calculates the area under the curve of the absolute difference time series between interlocutor time series. The length of the difference time series can be standardized the shortest number of exchanges present in the group using an internally defined resampling function, called with resample = TRUE. Spearman's rank correlation and area under the curve become less reliable for dyads under 30 exchanges.
#'
#' @name summarize_dyads
#' @param df_prep produced in the align_dyads function
#' @param custom_lags integer vector, should any lags be added in addition to -2, 0, 2
#' @param corr_type option for computing lagged correlations turn-by-turn covariance (default='Pearson')
#' @param sumdat_only default=TRUE, group and summarize data, two rows per conversation, one row for each participant, false will fill down summary statistics across all exchanges
#' @returns either:
#' - a grouped dataframe with summary data aggregated by converation (Event_ID) and participant if sumdat_only=T.
#' - the origoinal dataframe 'filled down' with summary data (e.g., AUC, turn-by-turn correlations) for each conversation is sumdat_only=F.
#' @importFrom DescTools AUC
#' @importFrom dplyr across
#' @importFrom dplyr bind_rows
#' @importFrom dplyr bind_cols
#' @importFrom dplyr distinct
#' @importFrom dplyr group_by
#' @importFrom dplyr filter
#' @importFrom dplyr first
#' @importFrom dplyr summarise
#' @importFrom dplyr left_join
#' @importFrom dplyr mutate
#' @importFrom dplyr rename_with
#' @importFrom dplyr rename_at
#' @importFrom dplyr select
#' @importFrom dplyr ungroup
#' @importFrom magrittr %>%
#' @importFrom stringr str_replace
#' @importFrom stringr str_c
#' @importFrom tidyr fill
#' @importFrom tidyr pivot_wider
#' @importFrom tidyr pivot_longer
#' @importFrom tidyr drop_na
#' @importFrom tidyselect starts_with
#' @importFrom tidyselect ends_with
#' @importFrom tidyselect contains
#' @importFrom tidyselect everything
#' @importFrom utils tail
#' @importFrom zoo na.approx
#' @export summarize_dyads

summarize_dyads <- function(df_prep, custom_lags = NULL, sumdat_only = TRUE, corr_type = 'Pearson') {
  # prevent note - no visible binding
  Lag <- Talked_First <- NULL
  # Validate correlation type at the start
  if (!corr_type %in% c("Pearson", "Spearman")) {
    stop("corr_type must be either 'Pearson' or 'Spearman'")
  }

  # Extract and Summarize Meta-Data - select Event_ID, Participant_ID, and non-numeric columns
  # excluding dimension columns and specific technical columns
  df_meta <- df_prep %>% dplyr::select(Event_ID, Participant_ID,
                                       -c(Exchange_Count, Turn_Count, Text_Prep, Text_Clean),
                                       -matches("^(emo_|lex_|phon_|sem_|df_)")) %>%
    # Select only factor and character columns (excluding numeric)
    dplyr::select(where(~ is.factor(.x) | is.character(.x)))

  # Check if there are any columns left besides Event_ID and Participant_ID
  meta_cols <- setdiff(names(df_meta), c("Event_ID", "Participant_ID"))

  # Only create summary if there are metadata columns to summarize
  if (length(meta_cols) > 0) {
    df_meta_sum <- df_meta %>%
      dplyr::group_by(Event_ID, Participant_ID) %>%
      dplyr::summarise(dplyr::across(.cols = everything(),
                                     .fns = ~ dplyr::first(.x),
                                     .names = "{.col}_summary"),
                       .groups = "drop")
  } else {
    df_meta_sum <- df_meta %>%
      dplyr::select(Event_ID, Participant_ID) %>%
      dplyr::distinct()
  }

  # Generate summary data by conversation and participant
  av_df <- df_prep %>%
    dplyr::select(Event_ID, Participant_ID, Exchange_Count, Turn_Count, Text_Clean, matches("^(sem_|lex_|emo_|phon_)")) %>%
    dplyr::group_by(Event_ID, Participant_ID) %>%
    dplyr::summarize(
      across(
        matches("^(sem_|lex_|emo_|phon_)"),
        ~ mean(.x, na.rm = TRUE),
        .names = "{.col}_mean"
      ),
      across(c(Exchange_Count, Turn_Count, Text_Clean), dplyr::first),
      .groups = "drop"
    ) %>%
    dplyr::select(Event_ID, Participant_ID, contains("mean")) %>%
    tidyr::pivot_longer(
      cols = matches("^(sem_|lex_|emo_|phon_).*_mean"),
      names_to = "Dimension",
      names_pattern = "(.*)_mean",
      values_to = "Dimension_Mean"
    )

  # Clean up factors
  df_prep <- droplevels(df_prep)

  # Add participant pairs
  df_prep <- df_prep %>%
    dplyr::group_by(Event_ID) %>%
    dplyr::mutate(Participant_Pair = paste(sort(unique(Participant_ID)), collapse = "---")) %>%
    dplyr::ungroup()

  # Compute AUC and lag correlations
  auc_df <- compute_auc(df_prep = df_prep) %>% dplyr::select(c("Event_ID", contains("AUC")))

  user_lags <- c(-2, 0, 2)
  if (!is.null(custom_lags)) {
    user_lags <- sort(unique(append(user_lags, custom_lags)))
  }

  # Pass corr_type to compute_lagcorr - this is the key modification
  covar_df <- compute_lagcorr(df_prep = df_prep, lags = user_lags, corr_type = corr_type)

  # Reshape AUC data
  auc_df_long <- auc_df %>%
    tidyr::pivot_longer(
      contains("AUC"),
      names_to = c("Dimension", "reshaped", "Lag"),
      names_pattern = "AUC_(.*)_(raw|scaled50)_(Immediate|Lag\\d|Lead\\d)",
      values_to = "AUC"
    ) %>%
    tidyr::pivot_wider( # pivot out the reshaped and lag columns
      names_from = c(reshaped, Lag),
      names_prefix = "AUC_",
      values_from = AUC
    )

  # Combine all data frames - with conditional join
  df_summarize <- av_df %>%
    dplyr::left_join(auc_df_long, by = c("Event_ID", "Dimension")) %>%
    dplyr::left_join(covar_df, by = c("Event_ID", "Dimension")) %>%
    dplyr::relocate(Talked_First, .after = Participant_ID) # move Talked_First to after Participant ID

  # Only join metadata if there were columns to summarize
  if (length(meta_cols) > 0) {
    df_summarize <- df_summarize %>%
      dplyr::left_join(df_meta_sum, by = c("Event_ID", "Participant_ID"))
  }

  # Remove row names
  row.names(df_summarize) <- NULL

  if (sumdat_only) {
    return(df_summarize)
  } else {
    # Join summary data back to original dataframe
    df_full <- df_prep %>%
      dplyr::left_join(df_summarize, by = c("Event_ID", "Participant_ID")) %>%
      # Fill down summary data within each Event_ID
      dplyr::group_by(Event_ID) %>%
      tidyr::fill(names(df_summarize)[!names(df_summarize) %in% c("Event_ID", "Participant_ID")], .direction = "downup") %>%
      dplyr::ungroup()

    return(df_full)
  }
}

# tests for summarize_dyads larger function

# data to input
one_rhyme <- ConversationAlign::NurseryRhymes_Prepped[ConversationAlign::NurseryRhymes_Prepped$Event_ID == "LittleLamb",]
one_rhyme <- droplevels(one_rhyme)

test_that("required column names are present", {
  # compute summarize dyads on single nursery rhyme
  output <- ConversationAlign::summarize_dyads(one_rhyme)
  # define list of required names and evaluate
  col_vec <- c("Event_ID", "Participant_ID", "Dimension", "Dimension_Mean", "AUC_raw", "AUC_scaled100", "Talked_First")
  col_vec <- c("Event_ID", "Participant_ID", "Talked_First", "Dimension", "Dimension_Mean", "AUC_raw_Immediate",
               "AUC_scaled50_Immediate", "AUC_raw_Lag1", "AUC_scaled50_Lag1", "TurnCorr_Lead2", "TurnCorr_Immediate", "TurnCorr_Lag2")
  expect_in(col_vec, colnames(output))
})

#' Sample Dyadic Interview Transcript: Marc Maron and Terry Gross Radio Interview 2013
#'
#' Text and talker information delineated, raw transcript, multiple lines per talker
#'
#' @format ## "MaronGross_2013"
#' A data.frame with 546 obs, 2 vars:
#' \describe{
#'   \item{text}{text from interview}
#'    \item{speaker}{speaker identity}
#'   ...
#' }
"MaronGross_2013"


' Sample Conversation Transcript: Nursery Rhymes
#'
#' Text and talker information delineated, 3 separate nursery rhymes, good for computing analytics and word counts
#'
#' @format ## "NurseryRhymes"
#' A data.frame with 100 observations, 2 vars:
#' \describe{
#'   \item{Event_ID}{factor 3 different simulated conversations}
#'    \item{Participant_ID}{fictional speaker names, 2 each conversation}
#'    \item{Text_Raw}{simulated language production, actually looped phrases from nursery rhymes}
#'   ...
#' }
"NurseryRhymes"


' Sample Conversation Transcript: Nursery Rhymes Prepped
#'
#' Text and talker information delineated, 3 separate nursery rhymes, good for computing analytics and word counts
#'
#' @format ## "NurseryRhymes_Prepped"
#' A data.frame with 1507 x 7 observations, 5 vars:
#' \describe{
#'   \item{Event_ID}{factor 3 different simulated conversations}
#'    \item{Participant_ID}{fictional speaker names, 2 each conversation}
#'    \item{Exchange_Count}{sequential numbering of exchanges by conversation, 1 exchange = 2 turns}
#'    \item{Turn_Count}{sequential numbering of turns by conversation}
#'    \item{Text_Clean}{content words}
#'     \item{emo_anger}{raw value of anger salience yoked to each word}
#'   ...
#' }
"NurseryRhymes_Prepped"

#' @importFrom utils globalVariables
NULL

utils::globalVariables(
  c("CA_orig_stops",
    "clean_me",
    "clean_dyads",
    "cor",
    "df",
    "df_prep",
    "df_read",
    "df_summarize",
    "Exchanges",
    "Exchange_Count",
    'Event_ID',
    "freq_lg10",
    "letter_count",
    "lex_freqlg10",
    "lex_n_morphemes",
    "lookup_Jul25",
    "measure",
    "MIT_Stops",
    "morpheme_count",
    "n_participants",
    "participant_var",
    'Participant_ID',
    "Participant_Pair",
    "phon_n_lett",
    "phon_nsyll",
    "reshaped",
    "score",
    "setNames",
    "SMART_stops",
    "stdev",
    "sumdat",
    "sumdat_only",
    "summary_statistic",
    'switch_mark',
    "syllable_count",
    "Temple_stops25",
    "Text_Clean",
    "Text_Prep",
    "Text_Raw",
    "Turn_Count",
    "value",
    "word",
    "words_per_turn_clean",
    "words_per_turn_raw"
  ))

#' compute_auc
#'
#' internal function that computes two indices of global alignment (auc) between conversation partners for each dyad
#' @name compute_auc
#' @importFrom dplyr bind_rows
#' @importFrom dplyr bind_cols
#' @importFrom dplyr left_join
#' @importFrom dplyr mutate
#' @importFrom dplyr lag
#' @importFrom dplyr select
#' @importFrom dplyr group_by
#' @importFrom dplyr ungroup
#' @importFrom dplyr summarize
#' @importFrom dplyr first
#' @importFrom dplyr rename_with
#' @importFrom dplyr across
#' @importFrom dplyr everything
#' @importFrom tidyselect contains
#' @importFrom tidyr pivot_wider
#' @importFrom tidyr fill
#' @importFrom magrittr %>%
#' @importFrom zoo na.approx
#' @importFrom DescTools AUC
#' @returns
#' nothing - internal function used for intermediary computation piped into summarize_dyads function
#' @keywords internal
#' @noRd

compute_auc <- function(df_prep, verbose = TRUE) {
  # additional_lags can be added as a parameter, and will accept a vector of ints
  additional_lags = c(1)
  # define the internal functions here:
  # function to calculate AUC for single time series, return NA if there is an error
  calculate_auc <- function(domain_ts, doc_name, dimension, verbose) {
    tryCatch({
      # if time series has fewer points than the threshold, fill with NA
      if (max(domain_ts$Exchange_Count) < 3) { # hard coded to three exchanges
        #create a single row, single column dataframe with one empty value to fill in the AUC
        doc_domain_auc_df <- data.frame(domain_auc = as.double(NA),
                                        Exchanges = max(domain_ts$Exchange_Count))
      }
      else {
        domain_ts <- data.frame(domain_ts) #make single emotion Time series a data frame
        # na.rm is enabled, which will drop any NA values, so any differences where one of the original scores=NA
        domain_auc <- DescTools::AUC(x = domain_ts[,1], y = domain_ts[,2],
                                     method = "trapezoid", na.rm = T)
        doc_domain_auc_df <- data.frame(domain_auc,
                                        Exchanges = max(domain_ts$Exchange_Count)) #make data frame of AUC, replicated once
      }
      doc_domain_auc_df
    },
    error = function(e) {
      if (verbose) {
        message(paste("Results for dAUC will be filled with NA.\n\tTranscript:",
                      doc_name, "\n\tDimension:", dimension))
      }
      # fill the result cell with NA
      doc_domain_auc_df <- data.frame(domain_auc = as.double(NA),
                                      Exchanges = max(domain_ts$Exchange_Count))
      doc_domain_auc_df
    })
  } # end of ccalculate_auc()

  # selects align_var by grepping on possible prefixes of dimensions
  align_var <- grep("^(emo_|lex_|sem_|phon_)", colnames(df_prep), value = TRUE, ignore.case = TRUE)

  # split the data frame into a list by event id
  df_list <- split(df_prep, f = df_prep$Event_ID)

  # create a list of the first speaker in each dyad (S1) - for appending at end
  s1_col <- df_prep %>%
    dplyr::group_by(Event_ID) %>%
    dplyr::summarize(S1 = unique(Participant_ID)[1])

  # iterate over each event, replacing participant names with a transient filler variable
  df_list_speakvar <- lapply(df_list, function(df){
    svec <- unique(as.character(df$Participant_ID))
    df <- df %>% dplyr::mutate(participant_var = ifelse(as.character(Participant_ID) == svec[1], "S1", "S2"),
                               Participant_Pair = paste(sort(svec), collapse = "---"))
  })

  df_speakvar <- dplyr::bind_rows(df_list_speakvar)

  # group by turn then take the average score for each turn count, then pivot on pids
  df_wide <- df_speakvar %>% dplyr::group_by(Event_ID, Exchange_Count, Participant_ID) %>%
    dplyr::summarise(dplyr::across(tidyselect::contains(align_var), ~ mean(.x, na.rm = TRUE)),
                     participant_var = dplyr::first(participant_var), Participant_Pair = dplyr::first(Participant_Pair),
                     .groups = "drop") %>%
    tidyr::pivot_wider(names_from = c("participant_var"), values_from = any_of(align_var))

  # if there is only one aligned variable manually add that variable name to participant columns
  if (length(align_var) == 1){
    colnames(df_wide)[which(colnames(df_wide) %in% c("S1", "S2"))] = paste(align_var[1], colnames(df_wide)[which(colnames(df_wide) %in% c("S1", "S2"))], sep = "_")
  }

  # should remove uneven final exchanges here - need to grab the last exchange count of each row and if it is uneven, remove it.
  for (eid in unique(as.character(df_wide$Event_ID))){
    final_exc_rows <- df_wide[which(df_wide$Event_ID == eid & df_wide$Exchange_Count == max(df_wide$Exchange_Count[which(df_wide$Event_ID == eid)])),]
    if (nrow(final_exc_rows) == 1){
      df_wide <- df_wide[-which(df_wide$Event_ID == eid & df_wide$Exchange_Count == max(df_wide$Exchange_Count[which(df_wide$Event_ID == eid)])),]
    }
  }

  # here is the big iteration:
  split_pid_df_list <- sapply(c("S1", "S2"), function(temp_pid){
    # grab the columns that are needed for grouping or contain just on participant's scores
    p_df <- df_wide %>%
      dplyr::select(c("Event_ID", "Exchange_Count", "Participant_Pair") | c(ends_with(paste0("_", temp_pid))))

    # take the first column that matches an aligned variable
    first_var_col <- p_df[,which(colnames(p_df) %in% paste(align_var, temp_pid, sep = "_"))[1]]
    # index the rows that are either not NA or are NaN values (numbers and NaN)
    p_df <- p_df[which(is.na(first_var_col) == FALSE | is.nan(unlist(first_var_col)) == TRUE),]
    p_df_list <- split(p_df, f = p_df$Event_ID)

    p_df_altered <- dplyr::bind_rows(p_df_list)
    # interpolate here - fill forward for end, fill back for front, linear interpolation for the middle
    # grouped by event id, then each function is run across all the Time series individually

    p_df_interp <- p_df_altered %>%
      dplyr::group_by(Event_ID) %>% # interpolates all missing values between two points
      dplyr::mutate(dplyr::across(tidyselect::contains(align_var),
                                  ~ zoo::na.approx(.x, na.rm = FALSE))) %>%
      tidyr::fill(tidyselect::contains(align_var), .direction = "downup") %>%
      dplyr::ungroup()

    # wrap the data frame in a list to preserve structure (lapply will pull it into a list of lists)
    df_interp_names_list <- list(p_df_interp)
  })
  # join the two participant data frame together by dyad and Exchange_Count
  widedf <- dplyr::left_join(split_pid_df_list[[1]], split_pid_df_list[[2]], by = c("Event_ID", "Participant_Pair", "Exchange_Count"))

  # create a list of wide data frames
  wide_df_list <- list(widedf)
  # get the length of the wide df list before adding lags - for naming columns
  start_wide_df_length <- length(wide_df_list)
  # if given, lag time series in widedf and add to list
  if (length(additional_lags > 0)) {
    # iteration for additional
    for (lag in additional_lags){
      lagdf <- widedf
      val <- abs(lag)
      # if positive lag all S1, if negative, lag all S2
      if (lag > 0) {
        lagdf <- widedf %>%
          dplyr::group_by(Event_ID) %>%
          dplyr::mutate(dplyr::across(tidyselect::contains("S1"), ~dplyr::lag(., n = val))) %>%
          dplyr::ungroup()
      }
      else if (lag < 0) {
        lagdf <- widedf %>%
          dplyr::group_by(Event_ID) %>%
          dplyr::mutate(dplyr::across(tidyselect::contains("S2"), ~dplyr::lag(., n = val))) %>%
          dplyr::ungroup()
      }
      wide_df_list[[length(wide_df_list)+1]] <- lagdf
    }
  }

  # iterate over the full computation of dAUC for each lag
  list_of_each_dauc_df <- lapply(seq_along(wide_df_list), function(i) {
    widedf <- wide_df_list[[i]]

    #iterate over each aligned dimension, selecting only the scores for that dimension and pulling a difference value and subbing it in for the actual values
    for (dimension in align_var){
      both_participant_cols <- widedf %>%
        dplyr::select(starts_with(dimension))
      absdiffcol <- data.frame(dimension = abs(both_participant_cols[,1] - both_participant_cols[,2]))
      widedf[which(colnames(widedf) %in% paste(dimension, c("S1", "S2"), sep = "_"))] <- absdiffcol
    }

    long_diff_df <- widedf %>%
      tidyr::pivot_longer(cols = c(ends_with("_S1") | ends_with("_S2")),
                          names_to = c("dimension", "Participant_ID"),
                          names_pattern = "(.*)_([^_]+)$",
                          values_to = "score") %>% # pivot longer by dimension and pid
      # pivot each dimension to a column
      tidyr::pivot_wider(names_from = dimension, values_from = score) %>%
      dplyr::filter(Participant_ID == "S1") %>% # remove every second row per turn
      dplyr::select(-Participant_ID) # remove pid column
    # split the difference data frame into a list based on event id
    long_diff_df_list <- split(long_diff_df, f = long_diff_df$Event_ID)

    # grab the aligned dimensions as a vector to iterate over
    xdimensions <- colnames(long_diff_df)[which(colnames(long_diff_df) %in% align_var)]

    # NOTE: calculate_auc function is defined here

    domain_auc_list <- lapply(xdimensions, function(dimension){ #iterate over emotion
      # iterate over each document
      single_doc_auc <- lapply(long_diff_df_list, function(df_prep){

        domain_ts <- df_prep %>%
          dplyr::select(Exchange_Count,
                        tidyselect::contains(dimension)) # take dimension and time

        # put the function in here
        single_doc_domain_auc <- calculate_auc(domain_ts,
                                               doc_name = as.character(df_prep$Event_ID)[1],
                                               dimension = dimension,
                                               verbose = verbose)
      })
      #bind all docs AUCs for emotion into one column and add column prefix
      all_doc_domain_auc_df <- dplyr::bind_rows(single_doc_auc)
      # drop exchange column for all but first dimension
      if (dimension != xdimensions[1]) {
        all_doc_domain_auc_df <- data.frame(all_doc_domain_auc_df[,1])
      }
      colnames(all_doc_domain_auc_df)[1] <- paste("AUC", dimension, sep = "_")
      all_doc_domain_auc_df
    })

    all_domain_df <- dplyr::bind_cols(domain_auc_list, data.frame(Event_ID = names(long_diff_df_list))) #bind all columns of AUCs into one data frame
    # standardize each AUC to 50
    all_domain_df_s <- all_domain_df %>%
      dplyr::mutate(dplyr::across(tidyselect::contains("AUC"), ~ (50 * .x) / Exchanges)) %>%
      dplyr::select(-Exchanges)

    output_auc <- dplyr::left_join(all_domain_df, all_domain_df_s,
                                   by = "Event_ID", suffix = c("_raw", "_scaled50")) %>%
      dplyr::select(c(Event_ID, Exchanges, dplyr::everything()))


    # add lag to columns if there is a lag
    if (i > start_wide_df_length) {
      # get lag from index
      lag_index <- i - start_wide_df_length
      output_auc <- output_auc %>%
        dplyr::rename_with(~ paste0(., "_Lag",
                                    as.character(additional_lags[lag_index])),
                           tidyselect::starts_with("AUC"))
    }
    # if not the first iteration, remove event id and exchanges
    if (i > 1) {
      output_auc <- output_auc %>%
        dplyr::select(-c(Event_ID, Exchanges))
    }
    output_auc
  })

  # combine the data frames from different lagged time series
  combined_lag_df <- dplyr::bind_cols(list_of_each_dauc_df) %>%
    dplyr::mutate(Talked_First = s1_col$S1, .after = Exchanges)
  # sub out negative lag colnames for lead
  colnames(combined_lag_df) <- gsub("_Lag-", "_Lead", colnames(combined_lag_df))
  # set the names of zero lag columsn to immediate
  immediate_ind <- grep(".*(raw|scaled50)$", colnames(combined_lag_df), ignore.case = F)
  #print(immediate_ind)
  colnames(combined_lag_df)[immediate_ind] <- sapply(colnames(combined_lag_df)[immediate_ind], function(x){
    paste0(x, "_Immediate")
  })

  #colnames(combined_lag_df)[immediate_ind] <- new_names

  # throw warning if any dyads are fewer than 50 exchanges
  small_dyads <- combined_lag_df[which(combined_lag_df$Exchanges < 50), "Event_ID"]
  small_dyads <- unique(small_dyads)
  if (length(small_dyads) > 0) {
    warning(paste0("Some conversations are shorter than 50 exchanges (100 turns). ",
                   "It is recommended that conversations are longer than 50 exchanges. ",
                   "Affected conversations:\n",
                   paste(small_dyads, collapse = ", ")))
  }

  return(combined_lag_df)
}

# unit testing for computing area under the curve

# single dyad for testing
one_rhyme <- NurseryRhymes_Prepped[NurseryRhymes_Prepped$Event_ID == "LittleLamb",]
one_rhyme <- droplevels(one_rhyme)


test_that("auc formats properly", {
  # computed previously
  expected_auc_df <- data.frame(
    Event_ID = "LittleLamb", Exchanges = 50, Talked_First = as.factor("Mary"),
    AUC_emo_anger_raw_Immediate = 1.485925, AUC_emo_anger_scaled50_Immediate = 1.485925,
    AUC_emo_anger_raw_Lag1 = 1.4556, AUC_emo_anger_scaled50_Lag1 = 1.4556
  )
  expected_auc_df$Talked_First <- factor(expected_auc_df$Talked_First, levels = c("Dave", "Mary"))
  # compare to computed df
  expect_equal(ConversationAlign:::compute_auc(one_rhyme), expected_auc_df)
})

# remove all but first three exchanges
one_rhyme_small <- one_rhyme[one_rhyme$Exchange_Count < 3, ]

test_that("auc fills with na when under 3 exchanges", {
  expected_small_auc_df <- data.frame(
    Event_ID = "LittleLamb", Exchanges = 2, Talked_First = as.factor("Mary"),
    AUC_emo_anger_raw_Immediate = as.double(NA), AUC_emo_anger_scaled50_Immediate = as.double(NA),
    AUC_emo_anger_raw_Lag1 = as.double(NA), AUC_emo_anger_scaled50_Lag1 = as.double(NA)
  )
  expected_small_auc_df$Talked_First <- factor(expected_small_auc_df$Talked_First, levels = c("Dave", "Mary"))
  # compare to computed df
  expect_equal(ConversationAlign:::compute_auc(one_rhyme_small), expected_small_auc_df)

})

---
title: ConversationAlign_Step3_Summarize
subtitle: summarize_dyads()
author: Jamie Reilly, Ben Sacks, Ginny Ulichney, Gus Cooney, Chelsea Helion
date: "`r format(Sys.Date(), '%B %d, %Y')`"
show_toc: true
output:
  rmarkdown::html_vignette:
    toc: yes
vignette: |
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{ConversationAlign_Step3_Summarize}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r, message=FALSE, warning=F, echo=F}
# Load ConversationAlign
# library(devtools)
# devtools::install_github("https://github.com/Reilly-ConceptsCognitionLab/ConversationAlign.git")
library(ConversationAlign)
```

This is the final step where `ConversationAlign` will compute summary statistics including main effects and alignment statistics for the vectorized dataframe you produced using `prep_dyads()`. Users have several options for how to output their data, and these choices should be guided by your analysis strategy. For example, a linear mixed effects approach might involve modeling the rise and fall of values across turns. In contrast, a standard ANOVA would work on grouped summary data.  

- Main effects for each dimension of interest aggregated by Conversation (Event_ID) and Person (Participant_ID). For example, main effects of concreteness would involve aggregated means of all the concreteness values for words produced by Mary vs words produced by Dave in an individual conversation.
- dAUC_raw: difference area under the curve reflecting the difference between interlocutors at each turn on each dimension uncorrected for conversation length. For example, if Mary's concreteness at Exchange 1 was 8 (on a scale of 0-9), and Dave's concreteness on Exchange 1 was 4, the difference between Mary and Dave in this one-turn conversation would be 4. dAUC reflects area across all turns estimated using the trapezoidal rule.
- dAUC_scaled100:  normalized AUC value to 100 turns using proportional scaling. e.g., (Observed AUC/Turns Raw) = (Normalized AUC)/100).
- Lead_Corr2: Pearson or Spearman lagged correlation reflecting turn-by-turn covariance across partners for each specified dimension
- Lag_Corr2: Lead correlation
- Who_Talked_First: Interlocutor who started the conversation (needed for interepreting lead/lag stats)


<span style="color: darkred;">Arguments to ``summarize_dyads()`` include:</span> <br>
1) **df_prep**= dataframe created by ``prep_dyads()``function <br>
2) **custom_lags**=  default is NULL, any additional user-specified lagged correlations. will automatically produce lead of 2 turns, immediate response, lag of 2 turns for each dimension of interest. <br>
3) **sumdat_only**= boolean default is TRUE, produces grouped summary dataframe with averages by conversation and participant for each alignment dimension, FALSE retrains all of the original rows, filling down empty rows of summary statistics for the conversation (e.g., AUC) <br>
4) **corr_type**= default='Pearson', other option 'Spearman' for computing turn-by-turn correlations across interlocutors for each dimension of interest.

```{r, eval=T, warning=F, message=F, options(digits=3)}
MarySumDat <- summarize_dyads(df_prep = NurseryRhymes_Prepped, custom_lags=NULL, sumdat_only = TRUE, corr_type='Pearson') 
colnames(MarySumDat)
knitr::kable(head(MarySumDat, 15), format = "simple", digits = 3)
```

## Generating sham conversations
Some research questions would benefit from the use of conversations that control for some temporal effects. The function ``generate_shams()`` accepts the output of ``prep_dyads()`` and returns a data frame in the same structure with each interlocutor's time series randomly shuffled. Since the output has the same format as ``prep_dyads()`` output, it can easily be supplied to ``summarize_dyads()`` and compared to the real conversations.

<span style="color: darkred;">Arguments to ``generate_shams()`` include:</span> <br>
1) **df_prep**= dataframe created by ``prep_dyads()``function <br>
2) **seed**= numeric, a number to supply as a seed. This allows for reproducible results. <br>

```{r, eval=T, warning=F, message=F}
MaryShams <- generate_shams(df_prep = NurseryRhymes_Prepped, seed = 10)
MarySumDatShams <- summarize_dyads(df_prep = MaryShams, custom_lags=NULL, sumdat_only = TRUE, corr_type='Pearson') 
knitr::kable(head(MarySumDatShams, 15), format = "simple", digits = 3)
``` 

---
title: ConversationAlign_Step1_Read
subtitle: Read and Format Data for ConversationAlign
author: Jamie Reilly, Ben Sacks, Ginny Ulichney, Gus Cooney, Chelsea Helion
date: "`r Sys.Date()`"
show_toc: true
output:
  rmarkdown::html_vignette:
    toc: yes
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{ConversationAlign_Step1_Read}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r, message=FALSE, warning=F, echo=F}
# Load SemanticDistance
library(ConversationAlign)
```

# Reading data into R for ConversationAlign
Half the battle with R is getting your data imported and formatted. This is especially true for string data and working with text. `ConversationAlign` uses a series of sequential functions to import, clean, and format your raw data. You **MUST** run each of these functions. They append important variable names and automatically reshape your data. <br>

# Prepping your data for import
- `ConversationAlign` works **ONLY** on dyadic (i.e., two person) conversation transcripts. 
- Each transcript must nominally contain two colummns, one column should delineate the interlocutor (person who produced the text), and another column should contain the text itself.
- `ConversationAlign` contains an import function called `read_dyads()` that will scan a target folder for text samples.
- `read_dyads()` will import all of your transcripts into R and concatenate them into a single dataframe.
-  `read_dyads()` will append each transcript's filename as a unique identifier for that conversation. This is SUPER important to remember when analyzing your data.
- Store each of your individual conversation transcripts (`.csv`, `.txt`, `.ai`) that you wish to concatenate into a corpus in a folder. `ConversationAlign` will search for a folder called `my_transcripts` in the same directory as your script.  However, feel free to name your folder anything you like. You can specify a custom path as an argument to read_dyads()
- Each transcript must nominally contain two columns of data (Participant and Text). All other columns (e.g., meta-data) will be retained. 

## `read_dyads()`  
Here are some exampples of `read_dyads()` in action. There is only one argument to `read_dyads()`, and that is `my_path`. This is for supplying a quoted directory path to the folder where your transcripts live. Remember to treat this folder as a staging area!  Once you are finished with a set of transcripts and don't want them read into `ConversationAlign` move them out of the folder, or specify a new folder.  Language data tends to proliferate quickly, and it is easy to forget what you are doing. Be a CAREFUL secretary, and record your steps.

<span style="color: darkred;">Arguments to `read_dyads` include:</span>  <br>
1. **my_path**: default is 'my_transcripts', change path to your folder name<br>
```{r, eval=F, message=F, warning=F}
#will search for folder 'my_transcripts' in your current directory
MyConvos <- read_dyads()

#will scan custom folder called 'MyStuff' in your current directory, concatenating all files in that folder into a single dataframe
MyConvos2 <- read_dyads(my_path='/MyStuff')
```

## `read_1file()`
- Read single transcript already in R environment. We will use `read_1file()` to prep the Marc Maron and Terry Gross transcript. Look at how the column headers have changed and the object name (MaronGross_2013) is now the Event_ID (a document identifier), <br>

<span style="color: darkred;">Arguments to `read_1file` include:</span>  <br>
1. **my_dat**: object already in your R environment containing text and speaker information.
```{r, eval=F, message=F, warning=F}
MaryLittleLamb <- read_1file(MaronGross_2013)
#print first ten rows of header
knitr::kable(head(MaronGross_2013, 15), format = "pipe")
```
<br>

---
title: ConversationAlign_Step2_Prep
subtitle: prep_dyads()
author: Jamie Reilly, Ben Sacks, Ginny Ulichney, Gus Cooney, Chelsea Helion
date: "`r format(Sys.Date(), '%B %d, %Y')`"
show_toc: true
output:
  rmarkdown::html_vignette:
    toc: yes
vignette: |
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{ConversationAlign_Step2_Prep}
  %\VignetteEncoding{UTF-8}
---

```{r, message=FALSE, warning=F, echo=F}
# Load SemanticDistance
library(ConversationAlign)
```

# Cleaning, Formatting, Aligning Norms to Your Data
Lots of wild operations happen in this next step that transform your unstructured text to numeric time series objects aggregated by conversation and interlocutor. It is important that you have a handle on what `prep_dyads()` does and what processes such as lemmatization and stopword removal mean. <br>

``prep_dyads()`` uses numerous regex to clean and format the data your just read into R in the previous step. ``ConversationAlign`` applies an ordered sequence of cleaning steps on the road toward vectorizing your original text into a one-word-per row format. These steps include: converting all text to lowercase, expanding contractions, omitting all non-alphabetic characters (e.g., numbers, punctuation, line breaks). In addition to text cleaning, users guide options for stopword removal and lemmatization. During formatting ``prep_dyads()`` will prompt you to select up to three variables for computing alignment on. This works by joining values from a large internal lookup database to each word in your language transcript. ``prep_dyads()`` is customizable via the following arguments.

### Stopword removal
There are two important arguments regarding stopword removal. ``omit_stops`` specifies whether or not to remove stopwords. ``which_stopwords`` specifies which stopword list you would like to apply with the default being ``Temple_stops25``. The full list of choices is: ``none``, ``SMART_stops``, ``CA_orig_stops``. ``MIT_stops``, and ``Temple_stops25``. Stopword removal is an important, yet also controversial step in text cleaning.  <br>

### Lemmatization
``ConversationAlign`` calls the ``textstem`` package as a dependency to lemmatize your language transcript. This converts morphologiocal derivatives to their root forms. The default is lemmatize=T. Sometimes you want to retain language output in its native form. If this is the case, change the argument in clean_dyads to lemmatize=F. ``clean_dyads()`` outputs word count metrics pre/post cleaning by dyad and interlocutor. This can be useful if you are interested in whether one person just doesn't produce many words or produces a great deal of empty utterances. <br>

### Dimension Selection
This is where the magic happens. ``prep_dyads()`` will yoke published norms for >40 possible dimensions to every content word in your transcript (up to 3 at a time). This join is executed by merging your vectorized conversation transcript with a huge internal lexical database with norms spanning over 100k English words. ``prep_dyads()`` will prompt you to select anywhere from 1 to 3 target dimensions at a time. Enter the number corresponding to each dimension of interest separated by spaces and then hit enter (e.g., 10 14 19) ``ConversationAlign`` will append a published norm if available (e.g., concreteness, word length) to every running word in your transcript. These quantitative values are used in the subsequent ``summarize_dyads()`` step to compute alignment. <br>

# `prep_dyads()`
-Cleans, formats, and vectorizes conversation transwcripts to a one-word-per-row format
-Yokes psycholinguistic norms for up to three dimensions at a time (from <40 possible dimensions) to each content word.
-Retains metadata <br>

<span style="color: darkred;">Arguments to `prep_dyads`:</span> <br>
1) **dat_read**= name of the dataframe created during `read_dyads()` <br>
2) **omit_stops**= T/F (default=T) option to remove stopwords
3) **lemmatize**= lemmatize strings converting each entry to its dictionary form, default is `lemmatize=TRUE` <br>
4) **which_stoplist**= quoted argument specifying stopword list, options include ``none``, ``MIT_stops``, ``SMART``, ``CA_OriginalStops``, or ``Temple_stops25``. Default is ``Temple_stops25`` <br>
5) **remove_backchannel**= logical, should turns comprised entirely of stopwords be removed or preserved as NAs. NAs are filled in future steps. Defaults is ``FALSE``.

```{r, eval=F, message=F, warning=F}
#Example of running the function
NurseryRhymes_Prepped <- prep_dyads(dat_read=NurseryRhymes, lemmatize=TRUE, omit_stops=T, which_stoplist="Temple_stops25")
```

## Example of a prepped dataset 
This embedded as external data in the package with 'anger' values yoked to each word.
```{r}
knitr::kable(head(NurseryRhymes_Prepped, 20), format = "simple", digits=2)
```

---
title: ConversationAlign_Introduction
author: Jamie Reilly, Ben Sacks, Ginny Ulichney, Gus Cooney, Chelsea Helion
date: "`r format(Sys.Date(), '%B %d, %Y')`"
show_toc: true
slug: ConversationAlign
output:
  rmarkdown::html_vignette:
    toc: yes
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{ConversationAlign_Introduction}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

A good conversation is a cooperative endeavor where both parties modify the form and content of their own production to align with each other. This is a phenomenon known as alignment. People align across many dimensions including the words they choose and the affective tenor of their prosody. ``ConversationAlign`` measures dynamics of lexical use between conversation partners across more than 40 semantic, lexical, phonological, and affective dimensions. Before launching into your analyses, there are some important use caveats to consider.

## Caveats for Using ConversationAlign
- Language analyses often proliferate in complexity. Spend the extra time to keep careful records and a logical organization system (e.g., smart filenaming, variable keys, a formalized processing pipeline). 
- ``ConversationAlign`` only works on dyadic language transcripts (i.e., 2-person dialogues). 
- ``ConversationAlign`` does NOT parse turns automatically. The software will aggregate all words produced by one speaker across sentences and rows until a switch occurs in the 'speaker' column.
- ``ConversationAlign`` will strip punctuation and other special characters automatically. 
- ``ConversationAlign`` will split/vectorize your text into a one-word-per-row format, retaining all variable labels.
- ``ConversationAlign`` will retain all meta-data throughout text processing (e.g., timestamps, grouping variables). ``ConversationAlign`` is pretty good at detecting and repairing unconventional font encoding systems, but it will not catch everything, You will find all sorts of hidden junk when you copy/paste interview transcripts from random websites are from YouTube. Inspect your transcripts to make sure they are what you think they are before launching into a complex computational analysis. 

## Prepare your Transcripts Outside of the Package
- Save each conversation transcript as a separate file (``*.txt``, ``*.csv``, or Otter ``*.ai``).
- Be deliberate about your filenaming convention. Each transcript's filename will become its unique document identifier (Event_ID) when importing into ``ConversationAlign``. <br>
- Store all the transcripts you want to analyze in the same folder (e.g., '/my_transcripts).
- Your transcript folder and analysis scripts should ideally exist within the same directory.
- Each raw comversation transcript **MUST** nominally contain at least two columns, talker/interlocutor and text.  
- Name your talker/interlocutor column as 'Interlocutor', 'Speaker', or 'Participant' (not case sensitive).
- Name your text column as 'Text', 'Utterance', or 'Turn' (not case sensitive).

# Installation
Install and load the development version of `ConversationAlign` from [GitHub](https://github.com/) using the `devtools` package.
```{r, message=FALSE, warning=F}
# Load SemanticDistance
library(ConversationAlign)
```

## Calibaration Transcripts Included in `ConversationAlign`
``ConversationAlign``contains two sample conversation transcripts that are pre-load when you call the package. These are:
**MaronGross_2013:** Interview transcript of Marc Maron and Terry Gross on NPR (2013). <br>
**NurseryRhymes:** Three nursery rhymes looping same phrases formatted as conversations, cleaned, and aligned to illustrate how the formatting pipeline reshaopes conversation transcripts.

### NurseryRhymes
```{r, eval=T, message=F, warning=F}
knitr::kable(head(NurseryRhymes, 20), format = "simple")
str(NurseryRhymes)
```

### Maron-Gross Interview
Here's one from a 2013 NPR interview (USA) between Marc Maron and Terry Gross, titled [Marc Maron: A Life Fueled By 'Panic And Dread'](https://www.npr.org/transcripts/179014321).<br>
```{r}
knitr::kable(head(MaronGross_2013, 20), format = "simple")
str(MaronGross_2013)
```

# Caveat emptor
Any analysis of language comes with assumptions and potential bias. For example, there are some instances where a researcher might care about morphemes and grammatical elements such as 'the', 'a', 'and', etc..  The default for ConversationAlign is to omit these as stopwords and to average across all open class words (e.g., nouns, verbs) in each turn by interlocutor. There are some specific cases where this can all go wrong. Here are some things to consider: <br>

1. <span style="color:red;">Stopwords </span>: ``ConversationAlign`` omits stopwords by default applying a customized stopword list, ``Temple_Stopwords25``. [CLICK HERE](https://osf.io/dc5k7) to inspect the list. This stopword list includes greetings, idioms, filler words, numerals, and pronouns.

2. <span style="color:red;">Lemmatization </span>: The package will lemmatize your language transcripts by default. Lemmatization transforms inflected forms (e.g., standing, stands) into their root or dictionary entry (e.g., stand). This helps for yoking offline values (e.g., happiness, concreteness) to each word and also entails what NLP folks refer to as 'term aggregation'. However, sometimes you might NOT want to lemmatize. You can easily change this option by using the argument, "lemmatize=FALSE," to the clean_dyads function below. <br>

3. <span style="color:red;">Sample Size Issue 1: Exchange Count</span>: The program derives correlations and AUC for each dyad as metrics of alignment. For very brief conversations (<30 turns), the likelihood of unstable or unreliable estimates is high. <br>

4. <span style="color:red;">Sample Size Issue 2 </span>: matching to lookup database: ConversationAlign works by yoking values from a lookup database to each word in your language transcript. Some variables have lots of values characterizing many English words. Other variables (e.g., age of acquisition) only cover about 30k words. When a word in your transcript does not have a 'match' in the lookup datase, ConversationAlign will return an NA which will not go into the average of the words for that interlocutor and turn. This can be dangerous when there are many missing values. Beware! <br>

5. <span style="color:red;">Compositionality </span>: ConversationAlign is a caveman in its complexity. It matches a value to each word as if that word is an island. Phenomena like polysemy (e.g., bank) and the modulation of one word by an intensifier (e.g., very terrible) are not handled. This is a problem for many of the affective measures but not for lexical variables like word length. <br>


# Background and Supporting Materials

1. **Preprint** <br>
Our PsyArXiv preprint describing the method(s) in greater detail is referenced as:
Sacks, B., Ulichney, V., Duncan, A., Helion, C., Weinstein, S., Giovannetti, T., ... Reilly, J. (2025, March 12). *ConversationAlign: Open-Source Software for Analyzing Patterns of Lexical Use and Alignment in Conversation Transcripts*. to read our preprint. It was recently invited for revision at Behavior Rsearch Methods. We will update when/if eventually accepted there! <br>

2. **Variable Key for ConversationAlign** <br>
``ConversationAlign`` currently allows users to compute alignment dynamics across >40 different lexical, affective, and semantic dimensions.[Click Here to View Lookup Key](https://reilly-lab.github.io/Jamie_LookupVariableKey.html). <br>

# References
Lewis, David D., et al. (2004) "Rcv1: A new benchmark collection for text categorization research." Journal of machine learning research 5: 361-397.

---
title: ConversationAlign_Step4_Analytics
subtitle: corpus_analytics()
author: Jamie Reilly, Ben Sacks, Ginny Ulichney, Gus Cooney, Chelsea Helion
date: "`r format(Sys.Date(), '%B %d, %Y')`"
show_toc: true
output:
  rmarkdown::html_vignette:
    toc: yes
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{ConversationAlign_Step4_Analytics}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r, message=FALSE, warning=F, echo=F}
# Load SemanticDistance
library(ConversationAlign)
```

This is a helpful addition to `ConversationAlign` that will generate a variety of corpus analytics (e.g., word count, type-token-ratio) for your conversation corpus. The output is in a summary table that is readily exportable to to the specific journal format of your choice using any number of packages such as `flextable` or `tinytable`. <br>

Generate your corpus analytics on the dataframe you created with `prep_dyads`.

<span style="color: darkred;">Arguments to `corpus_analytics` include:</span> <br>
1) **dat_prep**= dataframe created by ``prep_dyads()``function <br>

```{r, eval=T, warning=F, message=F}
NurseryRhymes_Analytics <-  corpus_analytics(dat_prep=NurseryRhymes_Prepped)
knitr::kable(head(NurseryRhymes_Analytics, 15), format = "simple", digits = 2)
```



`

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{load_github_data}
\alias{load_github_data}
\title{Load all .rda files from a GitHub data folder into the package environment}
\usage{
load_github_data(
  repo = "Reilly-ConceptsCognitionLab/ConversationAlign_Data",
  branch = "main",
  data_folder = "data",
  envir = parent.frame()
)
}
\arguments{
\item{repo}{GitHub repository (e.g., "username/repo")}

\item{branch}{Branch name (default: "main")}

\item{data_folder}{Remote folder containing .rda files (default: "data/")}

\item{envir}{Environment to load into (default: package namespace)}
}
\value{
nothing, loads data (as rda files) from github repository needed for other package functions
}
\description{
Load all .rda files from a GitHub data folder into the package environment
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{MaronGross_2013}
\alias{MaronGross_2013}
\title{Sample Dyadic Interview Transcript: Marc Maron and Terry Gross Radio Interview 2013}
\format{
## "MaronGross_2013"
A data.frame with 546 obs, 2 vars:
\describe{
  \item{text}{text from interview}
   \item{speaker}{speaker identity}
  ...
}
}
\usage{
MaronGross_2013
}
\description{
Text and talker information delineated, raw transcript, multiple lines per talker
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_dyads.R
\name{prep_dyads}
\alias{prep_dyads}
\title{prep_dyads}
\usage{
prep_dyads(
  dat_read,
  lemmatize = TRUE,
  omit_stops = TRUE,
  which_stoplist = "Temple_stops25",
  remove_backchannel = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{dat_read}{dataframe produced from read_dyads() function}

\item{lemmatize}{logical, should words be lemmatized (switched to base morphological form), default is TRUE}

\item{omit_stops}{option to remove stopwords, default TRUE}

\item{which_stoplist}{user-specified stopword removal method with options including "none", "SMART", "MIT_stops", "CA_OriginalStops", or "Temple_Stopwords25".
"Temple_Stopwords25 is the default list}

\item{remove_backchannel}{logical, should turns that are full of stopwords (e.g., "Uhm yeah") be preserved as NAs or removed. Removal will 'squish' the turn before and after together into one. If NAs are preserved they are later interpolated.}

\item{verbose}{display detailed output such as error messages and progress (default is TRUE)}
}
\value{
dataframe with text cleaned and vectorized to a one word per-row format.
Lexical norms and metadata are appended to each content word. Cleaned text appears under a new column
called 'Text_Clean'. Any selected dimensions (e.g., word length) and metadata are also appended to each word along
with speaker identity, turn, and Event_ID (conversation identifier).
}
\description{
Cleans, vectorizes and appends lexical norms to all content words in a language corpus.
User guides options for stopword removal and lemmatization. User selects up to three psycholinguistic dimensions to yoke norms
on each content word in the original conversation transcript.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/corpus_analytics.R
\name{corpus_analytics}
\alias{corpus_analytics}
\title{corpus_analytics}
\usage{
corpus_analytics(dat_prep)
}
\arguments{
\item{dat_prep}{takes dataframe produced from the df_prep() function}
}
\value{
dataframe with summary statistics (mean, SD, range) for numerous corpus analytics (e.g., token count, type-token-ratio, word-count-per-turn) for the target conversation corpus. Summary data structured in table format for easy export to a journal method section.
}
\description{
Produces a table of corpus analytics including numbers of complete observations at each step, word counts, lexical diversity (e.g., TTR), stopword ratios, etc. Granularity of the summary statistics are guided by the user (e.g., by conversation, by conversation and speaker, collapsed all)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{NurseryRhymes}
\alias{NurseryRhymes}
\title{Text and talker information delineated, 3 separate nursery rhymes, good for computing analytics and word counts}
\format{
## "NurseryRhymes"
A data.frame with 100 observations, 2 vars:
\describe{
  \item{Event_ID}{factor 3 different simulated conversations}
   \item{Participant_ID}{fictional speaker names, 2 each conversation}
   \item{Text_Raw}{simulated language production, actually looped phrases from nursery rhymes}
  ...
}
}
\usage{
NurseryRhymes
}
\description{
Text and talker information delineated, 3 separate nursery rhymes, good for computing analytics and word counts
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_1file.R
\name{read_1file}
\alias{read_1file}
\title{read_1file}
\usage{
read_1file(my_dat)
}
\arguments{
\item{my_dat}{one conversation transcript already in the R environment}
}
\value{
a dataframe formatted with 'Event_ID', "Participant_ID", "Text_Raw" fields -- ready for clean_dyads()
}
\description{
Reads pre-formatted dyadic (2 interlocutor) conversation transcript already imported into your R environment.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize_dyads.R
\name{summarize_dyads}
\alias{summarize_dyads}
\title{summarize_dyads}
\usage{
summarize_dyads(
  df_prep,
  custom_lags = NULL,
  sumdat_only = TRUE,
  corr_type = "Pearson"
)
}
\arguments{
\item{df_prep}{produced in the align_dyads function}

\item{custom_lags}{integer vector, should any lags be added in addition to -2, 0, 2}

\item{sumdat_only}{default=TRUE, group and summarize data, two rows per conversation, one row for each participant, false will fill down summary statistics across all exchanges}

\item{corr_type}{option for computing lagged correlations turn-by-turn covariance (default='Pearson')}
}
\value{
either:
- a grouped dataframe with summary data aggregated by converation (Event_ID) and participant if sumdat_only=T.
- the origoinal dataframe 'filled down' with summary data (e.g., AUC, turn-by-turn correlations) for each conversation is sumdat_only=F.
}
\description{
Calculates and appends 3 measures for quantifying alignment. Appends the averaged value for each selected dimension by turn and speaker. Calculates and Spearman's rank correlation between interlocutor time series and appends by transcript. Calculates the area under the curve of the absolute difference time series between interlocutor time series. The length of the difference time series can be standardized the shortest number of exchanges present in the group using an internally defined resampling function, called with resample = TRUE. Spearman's rank correlation and area under the curve become less reliable for dyads under 30 exchanges.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_dyads.R
\name{read_dyads}
\alias{read_dyads}
\title{read_dyads}
\usage{
read_dyads(my_path = "my_transcripts")
}
\arguments{
\item{my_path}{folder of conversation transcripts in csv or txt format}
}
\value{
a dataframe where each individual conversation transcript in a user's directory has been concatenated.
read_dyads appends a unique document identifier to each conversation transcript appending its unique filename as a factor level to 'Event_ID'.
}
\description{
Reads pre-formatted dyadic (2 interlocutor) conversation transcripts from your machine. Transcripts must be either csv or txt format. IF you are supplying a txt file, your transcript must be formatted as an otter.ai txt file export. Your options for using csv files are more flexible. ConversationAlign minimally requires a csv file with two columns, denoting interlocutor and text. Each separate conversation transcript should be saved as a separate file. ConversationAlign will use the file names as a document ID. Within the read dyads function, set the my_path argument as the directory path to the local folder containing your transcripts on your machine (e.g., "my_transcripts"). Please see our github page for examples of properly formatted transcripts: https://github.com/Reilly-ConceptsCognitionLab/ConversationAlign
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_shams.R
\name{generate_shams}
\alias{generate_shams}
\title{generate_shams}
\usage{
generate_shams(df_prep, seed = NULL)
}
\arguments{
\item{df_prep}{Output dataframe of prep_dyads().}

\item{seed}{(Optional) a seed for reproducibility in random sampling}
}
\value{
A dataframe similar to prepped dyads, with each participant's time series randomly shuffled.
}
\description{
Generates a permutation of each individual dyad. Shuffled dyads may act as controls to their originals.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{NurseryRhymes_Prepped}
\alias{NurseryRhymes_Prepped}
\title{Text and talker information delineated, 3 separate nursery rhymes, good for computing analytics and word counts}
\format{
## "NurseryRhymes_Prepped"
A data.frame with 1507 x 7 observations, 5 vars:
\describe{
  \item{Event_ID}{factor 3 different simulated conversations}
   \item{Participant_ID}{fictional speaker names, 2 each conversation}
   \item{Exchange_Count}{sequential numbering of exchanges by conversation, 1 exchange = 2 turns}
   \item{Turn_Count}{sequential numbering of turns by conversation}
   \item{Text_Clean}{content words}
    \item{emo_anger}{raw value of anger salience yoked to each word}
  ...
}
}
\usage{
NurseryRhymes_Prepped
}
\description{
Text and talker information delineated, 3 separate nursery rhymes, good for computing analytics and word counts
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ConversationAlign-package.R
\docType{package}
\name{ConversationAlign-package}
\alias{ConversationAlign}
\alias{ConversationAlign-package}
\title{ConversationAlign: Process Text and Compute Linguistic Alignment in Conversation Transcripts}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Imports conversation transcripts into R, concatenates them into a single dataframe appending event identifiers, cleans and formats the text, then yokes user-specified psycholinguistic database values to each word. 'ConversationAlign' then computes alignment indices between two interlocutors across each transcript for >40 possible semantic, lexical, and affective dimensions. In addition to alignment, 'ConversationAlign' also produces a table of analytics (e.g., token count, type-token-ratio) in a summary table describing your particular text corpus.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/Reilly-ConceptsCognitionLab/ConversationAlign}
}

}
\author{
\strong{Maintainer}: Jamie Reilly \email{jamie_reilly@temple.edu} (\href{https://orcid.org/0000-0002-0891-438X}{ORCID})

Authors:
\itemize{
  \item Virginia Ulichney
  \item Ben Sacks
}

Other contributors:
\itemize{
  \item Sarah Weinstein [contributor]
  \item Chelsea Helion [contributor]
  \item Gus Cooney [contributor]
}

}
\keyword{internal}
